package grammars;

import java.util.*;

/**
 * Trida reprezentujici libovolnou vetnou formu slozenou ze symbolu
 *
 * @author Martin Kot
 * @version 1.0
 */
public class SententialForm{
  /**Pole symbolu tvorici vetnou formu*/
  protected Symbol sentence[];

  /**
   * Vytvori vetnou formu tvorenou danymi symboly. Symbol lambda bude jen, kdyz
   * bude vetna forma nulove delky.
   *
   * @param w Pole symbolu, ktere budou tvorit vetnou formu
   */
  public SententialForm(Symbol[] w){
    int j;
    if(w.length==0){
      sentence=new Symbol[1];
      sentence[0]=new Terminal(Grammar.lambda);
    }
    else{
      Vector v = new Vector();
      for(j=0;j<w.length;j++){
        if(!w[j].hasName(Grammar.lambda)){
          v.add(w[j]);
        }
      }
      if(v.size()!=0){
        sentence = (Symbol[])v.toArray(new Symbol[v.size()]);
      }
      else {
        sentence=new Symbol[1];
        sentence[0]=new Terminal(Grammar.lambda);
      }
    }
  }

  /**
   * Vytvori vetnou formu tvorenou jednim symbolem
   *
   * @param w Symbol, ktery bude tvorit vetnou formu
   */
  public SententialForm(Symbol w){
    sentence=new Symbol[1];
    if(w==null){
      sentence[0]=new Terminal(Grammar.lambda);
    }
    else{
      sentence[0]=w;
    }
  }

  /**
   * Vytvori kopii vetne formy
   *
   * @param sf1 Vetna forma, jejiz kopie se tvori
   */
  public SententialForm(SententialForm sf1){
    int i;
    sentence = new Symbol[sf1.length()];
    for(i=0;i<sf1.length();i++){
      sentence[i]=sf1.symbolAt(i);
    }
  }

  /**
   * Spoji dve vetne formy do nove
   *
   * @param sf1 Vetna forma, ktera bude po spojeni vlevo
   * @param sf2 Vetna forma, ktera bude po spojeni vpravo
   */
  public SententialForm(SententialForm sf1, SententialForm sf2){
    int j;
    Vector v = new Vector();
    for(j=0;j<sf1.length();j++){
      if(!sf1.symbolAt(j).hasName(Grammar.lambda)){
        v.add(sf1.symbolAt(j));
      }
    }
    for(j=0;j<sf2.length();j++){
      if(!sf2.symbolAt(j).hasName(Grammar.lambda)){
        v.add(sf2.symbolAt(j));
      }
    }
    if(v.size()!=0){
      sentence = (Symbol[])v.toArray(new Symbol[v.size()]);
    }
    else {
      sentence=new Symbol[1];
      sentence[0]=new Terminal(Grammar.lambda);
    }
  }

  /**
   * Spoji tri vetne formy do nove
   *
   * @param sf1 Vetna forma, ktera bude po spojeni vlevo
   * @param sf2 Vetna forma, ktera bude po spojeni uprostred
   * @param sf3 Vetna forma, ktera bude po spojeni vpravo
   */
  public SententialForm(SententialForm sf1, SententialForm sf2,
      SententialForm sf3){
    int j;
    Vector v = new Vector();
    for(j=0;j<sf1.length();j++){
      if(!sf1.symbolAt(j).hasName(Grammar.lambda)){
        v.add(sf1.symbolAt(j));
      }
    }
    for(j=0;j<sf2.length();j++){
      if(!sf2.symbolAt(j).hasName(Grammar.lambda)){
        v.add(sf2.symbolAt(j));
      }
    }
    for(j=0;j<sf3.length();j++){
      if(!sf3.symbolAt(j).hasName(Grammar.lambda)){
        v.add(sf3.symbolAt(j));
      }
    }
    if(v.size()!=0){
      sentence = (Symbol[])v.toArray(new Symbol[v.size()]);
    }
    else {
      sentence=new Symbol[1];
      sentence[0]=new Terminal(Grammar.lambda);
    }
  }

  /**
   * Slouzi ke zjisteni delky vetne formy
   *
   * @return Pocet symbolu vetne formy.
   */
  public int length(){
    if(sentence == null){
      return -1;
    }
    return sentence.length;
  }
  
  /**
   * Slouzi ke ziskani symbolu na urcite pozici ve vetne forme. Pokud je 
   * pozadovana pozice mimo hranice vetne formy, generuje se vyjimka 
   * IndexOutOfBoundsException. Neni ale deklarovana klauzuli throws, aby 
   * nemusela byt chytana, kdyz se predem delka zkontroluje.
   *
   * @param pos Pozice pozadovaneho symbolu. Pocita se od 0.
   * @return Symbol na dane pozici
   */
  public Symbol symbolAt(int pos){
    if(sentence==null){
      return null;
    }
    return sentence[pos];
  }

  /**
   * Hleda podformu vetne formy.
   * 
   * @param sw Hledana podforma
   * @return V pripade nalezeni je to pozice prvniho znaku podformy ve vetne
   * forme. Pri nenalezeni vraci -1.
   */
  public int indexOf(SententialForm sw){
    int i,j,k;
    boolean eq=true;
    for(i=0;i<sentence.length;i++){
      eq=true;
      for(j=0,k=i;j<sw.length();j++,k++){
        if(!sentence[k].equals(sw.symbolAt(j))){
          eq=false;
          break;
        }
      }
      if(eq==true){
        return i;
      }
    }
    return -1;
  }

  /**
   * Hleda vsechny vyskyty podformy ve vetne forme.
   * 
   * @param sw Hledana podforma
   * @return Pole pozic ve vetne forme, na kterych se nachazi prvni znak 
   * podformy. Neni-li nalezen zadny vyskyt, vraci pole delky 0.
   */
  public int[] allIndexOf(SententialForm sw){
    Vector v = new Vector();
    int i,j,k;
    boolean eq=true;
    int retval[];
    for(i=0;i<sentence.length;i++){
      eq=true;
      k=i;
      for(j=0;j<sw.length();j++){
        if(!sentence[k++].equals(sw.symbolAt(j))){
          eq=false;
          break;
        }
      }
      if(eq==true){
        v.add(new Integer(i));
      }
    }
    if(v.size()>0){
      retval=new int[v.size()];
    }
    else{
      return new int[0];
    }
    for(i=0;i<v.size();i++){
      retval[i]=((Integer)v.elementAt(i)).intValue();
    }
    return retval;
  }

  /**
   * Zjisti, jestli jedna vetna forma je podformou druhe
   *
   * @param w Forma, ve ktere se hleda
   * @param sw Hledana podforma
   * @return true, pokud se jedna o podformu. Jinak false.
   */
  public static boolean isSubword(SententialForm w, SententialForm sw){
    return (w.indexOf(sw)<0)?false:true;
  }

  /**
   * Slouzi k ziskani podformy z pozadovaneho rozmezi pozic
   *
   * @param begin Pozice prvniho znaku vracene podformy
   * @param end Pozice prvniho znaku, ktery jiz do vetne formy nebude zarazen
   * @return Pozadovana podforma
   */
  public SententialForm subword(int begin, int end){
    int i;
    if(begin > end){
      return null;
    }
    Symbol retval[] = new Symbol[end-begin];
    for(i=begin;i<end;i++){
      retval[i-begin]=sentence[i];
    }
    return new SententialForm(retval);
  }

  /**
   * Slouzi k ziskani podformy od urceneho zacatku do konce puvodni
   *
   * @param begin Pozice prvniho znaku vracene podformy
   * @return Pozadovana podforma
   */
  public SententialForm subword(int begin){
    return subword(begin, this.length());
  }

  /**
   * Prevede vetnou formu na retezec tvoreny jmeny symbolu oddelenymi mezerami
   *
   * @return Retezec reprezentujici vetnou formu
   */
  public String toString(){
    if(this.sentence==null){
      return "";
    }
    StringBuffer sb = new StringBuffer();
    for(int i=0; i<this.sentence.length; i++){
      if(i!=0){
        sb.append(" ");
      }
      sb.append(this.sentence[i].toString());
    }
    return sb.toString();
  }

  /**
   * Slouzi ke zjisteni, jestli je vetna forma tvorena pouze terminalnimi 
   * symboly
   *
   * @return true, pokud jsou vsechny symboly terminaly. Jinak false.
   */
  public boolean isTerminalWord(){
    for(int i=0;i<this.sentence.length;i++){
      if(!sentence[i].isTerminal()){
        return false;
      }
    }
    return true;
  }

  /**
   * Slouzi k porovnani vetne formy s jinou
   *
   * @param sf Vetna forma, se kterou se bude porovnavat
   * @return true, pokud maji symboly obou forme stejne nazvy a jsou ve stejnem
   * poradi. Jinak false.
   */ 
  public boolean isEqual(SententialForm sf){
    if(this.sentence.length!=sf.length()){
      return false;
    }
    for(int i=0; i<sentence.length; i++){
      if(!sentence[i].equals(sf.symbolAt(i))){
        return false;
      }
    }
    return true;
  }
}


