package grammars;


/**
 * Trida reprezentujici jeden krok derivace. Pamatuje si aktualni vetnou formu
 * a pravidlo, ktere je na ni pouzito. Navic ma promennou pro urceni, jestli
 * bylo pravidlo pouzito klasicky nebo ve smyslu testovani vyskytu.
 *
 * @author Martin Kot
 * @version 1.0
 */
public class Step {
  /**Aktualni vetna forma*/
  protected SententialForm form;
  /**Pouzite pravidlo*/
  protected Production prod;
  /**
   * Pokud bylo pravidlo pouzito ve smyslu testovani vyskytu, je hodnota true. 
   * Jinak je false.
   */
  protected boolean ac;

  /**
   * Vytvori krok derivace se vsemi parametry
   *
   * @param f Aktualni vetna forma
   * @param p Pouzite pravidlo
   * @param ac true, pokud bylo pravidlo pouzito ve smyslu testovani vyskytu. 
   * Jinak false.
   */
  public Step(SententialForm f, Production p, boolean ac) {
    this.prod=p;
    this.form=f;
    this.ac=ac;
  }

  /**
   * Vytvori krok derivace bez urceni pravidla
   *
   * @param f Aktualni vetna forma
   * @param ac true, pokud bylo pravidlo pouzito ve smyslu testovani vyskytu. 
   * Jinak false.
   */
  public Step(SententialForm f, boolean ac) {
    this.prod=null;
    this.form=f;
    this.ac=ac;
  }
 
  /**
   * Pomocny konstruktor. Vytvori krok derivace bez vsech parametru.
   */
  public Step() {
    this.prod=null;
    this.form=null;
    this.ac=false;
  }
 
  /**
   * Slouzi k ziskani pouziteho pravidla 
   *
   * @return Pouzite pravidlo
   */
  public Production getProduction(){
    return prod;
  }

  /**
   * Slouzi k ziskani vetne formy 
   *
   * @return Vetna forma
   */
  public SententialForm getForm(){
    return form;
  }

  /**
   * Slouzi k nastaveni vetne formy
   *
   * @param f Nova vetna forma
   */
  public void setForm(SententialForm f){
    this.form=f;
  }

  /**
   * Slouzi k nastaveni pouziteho pravidla
   *
   * @param f Nove pravidlo
   */
  public void setProduction(Production p){
    this.prod=p;
  }

  /**
   * Slouzi k zjisteni, jestli krok probehl v modu testovani vyskytu
   *
   * @return true, pokud aplikace pravidla v tomto kroku probehla ve smyslu
   * testovani vyskytu. Jinak false.
   */
  public boolean isAc(){
    return ac;
  }
}
