package grammars;
import java.util.*;

/**
 * Trida reprezentujici symbol abecedy
 *
 * @author Martin Kot
 * @version 1.0
 */
public class Symbol{
  /**Jmeno symbolu*/
  protected String symbol;
  /**Pocet pouziti v pravidlech gramatiky*/
  int useCount=0;

  /**
   * Pomocny konstruktor. Tvori symbol bez jmena.
   */
  public Symbol(){
    symbol = new String();
  }

  /**
   * Vytvori konstruktor s pozadovanym jmenem 
   *
   * @param s Jmeno symbolu
   */
  public Symbol(String s){
    symbol=s;
  }

  /**
   * Vytvori konstruktor s pozadovanym jmenem 
   *
   * @param s Jmeno symbolu tvorene jednim znakem
   */
  public Symbol(char c){
    char pole[]=new char[1];
    pole[0]=c;
    symbol=new String(pole);
  }

  /**
   * Porovna dva symboly
   * 
   * @param s Porovanavany symbol
   * @return true, pokud se symbol jmenuje stejne, jako ten v parametru
   */
  public boolean equals(Symbol s){
    if(symbol.compareTo(s.toString())==0){
      return true;
    }
    return false;
  }

  /**
   * Zjisti, jestli ma symbol pozadovane jmeno
   *
   * @param s Porovanavany retezec
   * @return true, pokud se symbol jmenuje tak, jak je pozadovano parametrem
   */
  public boolean hasName(String s){
    if(symbol.compareTo(s)==0){
      return true;
    }
    return false;
  }

  /**
   * Porovna dva symboly, jestli se stejne jmenuji
   * @param s1 Prvni porovnavany symbol
   * @param s2 Druhy porovnavany symbol
   * @return true, pokud se symboly jmenuji stejne
   */
  public static boolean areEqual(Symbol s1, Symbol s2){
    return s1.equals(s2);
  }

  /**
   * Prevede symbol na retezec tim, ze vrati jeho jmeno
   *
   * @return Jmeno symbolu
   */
  public String toString(){
    return symbol;
  }

  /**
   * Slouzi k zjisteni, jestli je symbol terminal. Predpoklada predefinovani
   * v podtridach.
   *
   * @return Vzdy vraci false
   */
  public boolean isTerminal(){
    return false;
  }

  /**
   * Slouzi k zjisteni, jestli je symbol neterminal. Predpoklada predefinovani
   * v podtridach.
   *
   * @return Vzdy vraci false
   */
  public boolean isNonterminal(){
    return false;
  }

  /**
   * Nastavi pocet vyskytu symbolu v pravidlech na pozadovanou hodnotu
   *
   * @param c Pozadovana hodnota poctu vyskytu
   */
  public void setUseCount(int c){
    this.useCount=c;
  }

  /**
   * Slouzi k ziskani poctu vyskytu symbolu v pravidlech
   *
   * @return Pocet vyskytu symbolu v pravidlech
   */
  public int getUseCount(){
    return this.useCount;
  }

  /**
   * Zmeni pocet vyskytu symbolu v pravidlech o pozadovanou hodnotu
   *
   * @param c Hodnota pricitana k poctu vyskytu symbolu v pravidlech
   */
  public int changeUseCount(int c){
    this.useCount+=c;
    return this.useCount;
  }
}
