package gui;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.event.*;
import java.util.*;
import grammars.*;
import java.io.*;


/**
 * Okno pro provadeni derivace. Umoznuje aplikovat pravidla, vracet se 
 * v derivaci zpet, prejit na editaci gramatiky. Navic je mozne derivaci 
 * nechat probihat automaticky. Na programovanou gramatiku je vazana jen tim,
 * ze pro editaci gramatiky vola PrgGramInputFrame. Pro dalsi typy gramatik
 * by se musela pred editaci dodat volba typu.
 *
 * @author Martin Kot
 * @version 1.0
 */
public class DerivationFrame extends JFrame implements Errorable{
  /**Vyska okna*/
  public static final int WINDOW_HEIGHT = 480;
  /**Sirka okna*/
  public static final int WINDOW_WIDTH = 600;

  /**Retezce nazvu menu*/
  protected static final String menuStrings[]=
     {"Derivace","Gramatika","Napoveda"};
  /**Retezce nazvu polozek v menu*/
  protected static final String menuItemsStrings[][]={
     {"Nova derivace","Aplikuj pravidlo", "Zpet", "Konec"},
     {"Nova","Nacist","Upravit","Ulozit"},
     {"Derivace","O programu"}};
  /**Pozice nazvu menu derivace*/
  protected static final int DERIVATION_MENU = 0;
  /**Pozice nazvu polozky menu nova derivace*/
  protected static final int NEW_ITEM = 0;
  /**Pozice nazvu polozky menu aplikuj pravidlo*/
  protected static final int APPLY_ITEM = 1;
  /**Pozice nazvu polozky menu zpet o krok v derivaci*/
  protected static final int BACK_ITEM = 2;
  /**Pozice nazvu polozky menu konec programu*/
  protected static final int END_ITEM = 3;

  /**Pozice nazvu menu gramatiky*/
  protected static final int GRAMMAR_MENU = 1;
  /**Pozice nazvu polozky menu nacist soubor*/
  protected static final int LOAD_ITEM = 1;
  /**Pozice nazvu polozky menu upravit gramatiku*/
  protected static final int CHANGE_ITEM = 2;
  /**Pozice nazvu polozky menu ulozit gramatiku do souboru*/
  protected static final int SAVE_ITEM = 3;

  /**Pozice nazvu menu napovedy*/
  protected static final int HELP_MENU = 2;
  /**Pozice nazvu polozky menu napoveda o derivaci*/
  protected static final int DER_ITEM = 0;
  /**Pozice nazvu polozky menu napoveda o programu*/
  protected static final int ABOUT_ITEM = 1;

  /**Okno napovedy*/
  protected HelpFrame hf=null;
  /**Gramatika, podle ktere se provadi derivace*/
  protected Grammar grammar=null;
  /**Okno pro editaci gramatiky*/
  protected PrgGramInputFrame inputFrame=null;
  /**Aktualni derivace*/
  protected Derivation derivation=null;
  /**Pravidla pouzitelna klasickym zpusobem na aktualni vetnou formu*/
  protected Production usableProds[]=null;
  /**Pravidla pouzitelna ve smyslu testovani vyskytu na aktualni vetnou formu*/
  protected Production acUsableProds[]=null;
  /**Cas v milisekundach mezi kroky pri automaticke derivaci*/
  protected int delay;
  /**Nastaveno na true, jestli ma byt prerusena automaticka derivace*/
  protected boolean stopped=true;
  /**
   * Pokud program bezi jako applet, je nastaven objekt tohoto appletu. Jinak
   * je nastaveno null.
   */
  protected GrammarSimulator applet=null;
  /**
   * Vlakno, ktere vola po urcenem case funkci pro dalsi krok automaticke 
   * derivace
   */
  protected Thread thread=null;
  /**Adresar, ze ktereho byl naposledy vybran soubor pro ukladani nebo cteni*/
  protected File directory=null;

  /**Posuvna lista pro vetnou formu*/
  protected JScrollPane formScroll = new JScrollPane();
  /**Textove pole pro vetnou formu*/
  protected JTextField formField = new JTextField();
  /**Posuvne listy pro seznam kroku derivace*/
  protected JScrollPane stepsScroll = new JScrollPane();
  /**Seznam kroku derivace*/
  protected JList stepsList = new JList();
  /**Tlacitko pro navrat v derivaci*/
  protected JButton backButton = new JButton();
  /**Tlacitko pro editaci gramatiky*/
  protected JButton gramUpdateButton = new JButton();
  /**Tlacitko pro novou derivaci*/
  protected JButton newButton = new JButton();
  /**Tlacitko pro aplikaci pravidla a zacatek i konec automatice derivace*/
  protected JButton applyButton = new JButton();
  /**Tlacitko pro editaci nove gramatiky*/
  protected JButton gramNewButton = new JButton();
  /**Vyber pravidla pro aplikaci*/
  protected JComboBox prodsBox = new JComboBox();
  /**Zaskrtavaci tlacitko pro vyber leftmost derivace*/
  protected JCheckBox leftCHBox = new JCheckBox();
  /**Zaskrtavaci tlacitko pro vyber automaticke derivace*/
  protected JCheckBox autoCHBox = new JCheckBox();
  /**Zaskrtavaci tlacitko pro umozneni vyberu nahodneho pravidla*/
  protected JCheckBox randomCHBox = new JCheckBox();
  /**
   * Textove pole, do nehoz se pise vyskyt leve strany pravidla, ktery ma byt
   * prepsan.
   */
  protected JTextField appearField = new JTextField();
  /**Popisek k appearField*/
  protected JLabel appearLabel = new JLabel();
  /**Textove pole se zpozdenim mezi kroky derivace*/
  protected JTextField delayField = new JTextField();
  /**Popisek k delayField*/
  protected JLabel delayLabel = new JLabel();
  /**Panel pro vsechna tlacitka*/
  protected JPanel buttonPanel = new JPanel();
  /**Panel pro delayField a delayLabel*/
  protected JPanel delayPanel = new JPanel();
  /**Panel pro zaskrtavaci tlacitka*/
  protected JPanel chBoxPanel = new JPanel();
  /**Panel pro appearField a appearLabel*/
  protected JPanel appearPanel = new JPanel();
  /**Layout pro panel s tlacitky*/
  protected GridLayout buttonsLayout = new GridLayout();
  /**Layout pro panel se zaskrtavacimi tlacitky*/
  protected GridLayout chBoxLayout = new GridLayout();
  /**Popisek k formField*/
  protected JLabel formLabel = new JLabel();
  /**Popisek k prodsBox*/
  protected JLabel prodLabel = new JLabel();
  /**Polozky menu*/
  protected JMenuItem menuItems[][];

  /**
   * Vytvori okno se vsemi prvky. Umoznuje i ukladani a nacitani souboru
   */
  public DerivationFrame() {
    try {
      makeFrame();
    }
    catch(Exception e) {
      e.printStackTrace();
    }
  }

  /**
   * Vytvori okno se vsemi prvky. Parametrem se muze umoznit nebo zakazat 
   * ukladani do souboru.
   */
  public DerivationFrame(GrammarSimulator ap) {
    this.applet=ap;
    try {
      makeFrame();
    }
    catch(Exception e) {
      e.printStackTrace();
    }
  }

  /**
   * Slouzi k nastaveni adresare, ve kterem byl naposledy vybran soubor
   * 
   * @param f Naposledy pouzity adresar
   */
  public void setDirectory(File f){
    this.directory=f;
  }

  /**
   * Slouzi k ziskani adresare, ve kterem byl naposledy vybran soubor
   * 
   * @return Naposledy pouzity adresar
   */
  public File getDirectory(){
    return this.directory;
  }

  /**
   * Slouzi ke zjisteni, jestli je okno soucast appletu nebo aplikace
   *
   * @return true, pokud se jedna o applet.
   */
  public boolean isApplet(){
    if(applet!=null){
      return true;
    }
    else{ 
      return false;
    }
  }

  /**
   * Slouzi ke zmene gramatiky, podle ktere se derivace provadi. Soucasne 
   * prerusi pripadnou automatickou derivaci a vytvori novou derivaci.
   *
   * @param g Nova gramatika
   */
  public void setGrammar(Grammar g){
    if(this.applyButton.getText().startsWith("Stop")){
      this.stopButtonAction();     //Preruseni automaticke derivace
    }
    this.grammar=g;
    //Umozni volbu vseho, co je vazano na existenci gramatiky
    this.prodsBox.setEnabled(true);
    this.applyButton.setEnabled(true);
    this.menuItems[DERIVATION_MENU][APPLY_ITEM].setEnabled(true);
    this.gramUpdateButton.setEnabled(true);
    this.menuItems[GRAMMAR_MENU][CHANGE_ITEM].setEnabled(true);
    this.menuItems[GRAMMAR_MENU][SAVE_ITEM].setEnabled(true);
    this.newButton.setEnabled(true);
    this.menuItems[DERIVATION_MENU][NEW_ITEM].setEnabled(true);
    this.derivation = new Derivation(grammar); //Nova derivace
    setSteps();  //Nastavi pole s kroky derivace
  } 

  /**
   * Nastavi aplikovatelna pravidla do seznamu, ze ktereho se vybiraji. 
   */
  private void setProds(){
    int i,j;
    if(grammar==null){
      return;
    }
    this.usableProds=this.derivation.usableProductions(
      this.leftCHBox.isSelected());  //Pravidla pouzitelna klasicky
    this.acUsableProds=this.derivation.acUsableProductions(
      this.leftCHBox.isSelected());  //Pravidla pouzitelna ve smyslu testovani
                                     //vyskytu
    this.prodsBox.removeAllItems();  //Odstrani stare polozky
    for(i=0; i<this.usableProds.length; i++){ //Prida klasicky aplikovatelna
      this.prodsBox.addItem(usableProds[i].toListItem());
    }
    for(j=0; j<this.acUsableProds.length; j++){  //Prida ac aplikovatelna
      StringBuffer sb = new StringBuffer("ac: ");
      sb.append(acUsableProds[j].toListItem());
      this.prodsBox.addItem(sb.toString());
    }
    if(derivation.isTerminalWord() || (i+j)==0){  //Nejsou aplikovatelna
                                                 //nebo je jiz terminalni slovo
      if(stopped){                               //Nebyla automaticka derivace
        this.applyButton.setEnabled(false);
        this.menuItems[DERIVATION_MENU][APPLY_ITEM].setEnabled(false);
      }
      else {                                    //Byla automaticka derivace
        stopped=true;
        this.unsetAutoConf();
      }
    }
    else{                                       //Derivace muze pokracovat
      this.applyButton.setEnabled(true);
      this.menuItems[DERIVATION_MENU][APPLY_ITEM].setEnabled(true);
    }
  }

  /**
   * Nastavi seznam kroku derivace, aplikovatelna pravidla a pri prvnim
   * kroku zakaze pouziti tlacitka zpet.
   */
  private void setSteps(){
    String s[];

    if(grammar.getStart()==null){ 
      return;
    }
    s=this.derivation.stepsToList(); //Vytvori z pravidel retezce
    this.stepsList.setListData(s);   //Nastavi retezce do seznamu
    if(s.length<=1){                 //Pri prvnim kroku zakaze volbu zpet
      this.backButton.setEnabled(false);
      this.menuItems[DERIVATION_MENU][BACK_ITEM].setEnabled(false);
    }
    this.formField.setText(derivation.getForm().toString());
    this.stepsScroll.getVerticalScrollBar().setValue(
      stepsScroll.getVerticalScrollBar().getMaximum());
    this.setProds();                //Nastavi i aplikovatelna pravidla
  }
  
  /**
   * Slouzi k navratu z chyboveho okna. Povoli pouzivani prvku v okne. 
   *
   * @param message Cislo chyboveho hlaseni, ktere bylo zobrazeno.
   */
  public void takeControl(int message){
    this.setEnabled(true);
    this.show();
  }

  /**
   * Vytvori v okne vsechny prvky a nastavi posluchace udalosti.
   */
  private void makeFrame(){
    Dimension dim = Toolkit.getDefaultToolkit().getScreenSize();
    this.setSize(WINDOW_WIDTH,WINDOW_HEIGHT);
    this.setLocation((dim.width - WINDOW_WIDTH)/2,  
      (dim.height - WINDOW_HEIGHT)/2);   //Vycentruje okno doprostred obrazovky
    this.setTitle("Simulace derivace");
    this.getContentPane().setLayout(null);
    this.addWindowListener(new WindowAdapter(){  //Posluchac udalosti okna
      public void windowClosing(WindowEvent e){  //Pri zavreni okna
        if(thread!=null){                    //Ukonci paralelni vlakno
          thread.interrupt();
          thread=null;
        }
        dispose();
        if(applet!=null){
          applet.stop();
          applet.destroy();                  //Ukonci applet
        } 
        else{
          try{
            System.exit(0);                  //Ukonci aplikaci
          }
          catch(java.security.AccessControlException except){
          }  //Pokud bezi jako applet, generuje exit() bezpecnostni vyjimku
        }
      }
    });

    formScroll.setHorizontalScrollBarPolicy(
        JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS); //Posuvna lista u vetne formy
                                                  //bude vzdy
    formScroll.setVerticalScrollBarPolicy(        //a horizontalni nikdy
	JScrollPane.VERTICAL_SCROLLBAR_NEVER);
    formScroll.setBounds(new Rectangle(160, 100, 420, 50));
    stepsScroll.setBounds(new Rectangle(20, 220, 360, 180));

    //Jen jeden krok muze byt vybran soucasne pro navrat v derivaci
    stepsList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    stepsList.setToolTipText(
      "Kroky derivace ve tvaru \"vetna forma => pouzite pravidlo\"");

    backButton.setEnabled(false);  //Na zacatku se nemuze vracet
    backButton.setToolTipText("Zpet k vybranemu kroku derivace, nebo o jeden krok");
    backButton.setText("Zpet");
    backButton.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {  //Pri stisku tlacitka zpet
        backButtonAction();
      }
    });

    gramUpdateButton.setEnabled(false); //Na zacatku gramatika neni definovana
                                        //a nemuze se tedy menit
    gramUpdateButton.setToolTipText("Zmena gramatiky");
    gramUpdateButton.setText("Upravit gramatiku");
    gramUpdateButton.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) { 
        gramUpdateButtonAction(); //Pri stisku tlacitka upravit gramatiku
      }
    });

    newButton.setEnabled(false); //Neni urcena gramatika, nemuze se menit 
                                 //derivace
    newButton.setToolTipText("Nova derivace");
    newButton.setText("Nova derivace"); 
    newButton.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        newButtonAction();     //Pri stisku tlacitka nova derivace
      }
    });

    applyButton.setEnabled(false);  //Neni gramatika, neni co aplikovat
    applyButton.setToolTipText("Uziti pravidla k derivaci");
    applyButton.setText("Aplikuj");
    applyButton.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        if(applyButton.getText().startsWith("S")){
          stopButtonAction();  //Stisk tlacitka stop
        }
        else {
          applyButtonAction(); //Stisk tlacitka aplikuj
        }
      }
    });

    prodsBox.setEnabled(false);//Neni gramatika, neni zadne aplikovatelne prav.
    prodsBox.setBounds(new Rectangle(160, 160, 420, 25));
    prodsBox.setToolTipText(
      "Pravidla pouzitelna v tomto kroku. Vybrane bude aplikovano.");
    prodLabel.setText("Pouzit pravidlo:");
    prodLabel.setBounds(new Rectangle(20, 160, 130, 30));

    leftCHBox.setToolTipText("Vybira pravidlo s levou stranou nejvice vlevo ve "
    +"vetne forme");
    leftCHBox.setSelected(false); //Na zacatku je leftmost zvoleno
    leftCHBox.setText("Leftmost");
    leftCHBox.addChangeListener(new javax.swing.event.ChangeListener() {
      public void stateChanged(ChangeEvent e) {
        leftCHBoxStateChanged();  //Zmena zaskrtnuti tlacitka
      }
    });

    autoCHBox.setToolTipText("Provadi automaticky derivaci, je-li k "+
      "dispozici jen jedno pravidlo");
    autoCHBox.setSelected(true); //Na zacatku je zvolena automaticka derivace
    autoCHBox.setText("Automaticky");
    autoCHBox.addChangeListener(new javax.swing.event.ChangeListener() {
      public void stateChanged(ChangeEvent e) {
        autoCHBoxStateChanged(); //Zmena zaskrtnuti tlacitka automaticky
      }
    });

    randomCHBox.setToolTipText("Nahodne vybira pravidlo pri automaticke "+
      "derivaci");
    randomCHBox.setText("Nahodne");

    appearField.setEnabled(false);
    appearField.setToolTipText("Poradi vyskytu symbolu, ktery ma "+
      "byt derivovan");
    appearField.setText("1"); //Na zacatku je zvolen prvni vyskyt
    appearField.setColumns(3);
    appearLabel.setText("Vyskyt");

    delayField.setToolTipText("Zpozdeni mezi kroky derivace v milisekundach");
    delayField.setText("1000"); //Na zacatku je zvoleno zpozdeni 1000ms
    delayField.setColumns(5);   
    delayLabel.setText("Zpozdeni");

    gramNewButton.setToolTipText("Umozni zadani nove gramatiky");
    gramNewButton.setText("Nova gramatika");
    gramNewButton.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        gramNewButtonAction(); //Stisk tlacitka nova derivace
      }
    });

    buttonPanel.setEnabled(false);
    buttonPanel.setBounds(new Rectangle(400, 220, 180, 180));
    buttonPanel.setLayout(buttonsLayout);
    buttonsLayout.setColumns(1);  //Usporadani tlacitek bude v jednom sloupci
    buttonsLayout.setRows(5);     //a peti radcich 
    buttonsLayout.setVgap(10);    //s mezeou 10 bodu mezi nimi

    delayPanel.setEnabled(false);
    delayPanel.add(delayLabel, null);
    delayPanel.add(delayField, null);
    delayPanel.setAlignmentX(JPanel.LEFT_ALIGNMENT);
    delayLabel.setAlignmentX(JLabel.LEFT_ALIGNMENT);

    appearPanel.add(appearLabel, null);
    appearPanel.add(appearField, null);

    chBoxPanel.setEnabled(false);
    chBoxPanel.setBounds(new Rectangle(40, 30, 560, 40));
    chBoxPanel.setLayout(chBoxLayout);
    chBoxLayout.setColumns(3);
//    chBoxLayout.setHgap(20);
    chBoxLayout.setRows(2);
    chBoxPanel.setSize(500,50);
    chBoxPanel.add(autoCHBox, null);
    chBoxPanel.add(leftCHBox, null);
    chBoxPanel.add(randomCHBox, null);
    chBoxPanel.add(delayPanel, null);
    chBoxPanel.add(appearPanel, null);

    formField.setEditable(false); //Vetnou formu nelze menit
    formField.setToolTipText("Aktualni vetna forma");
    formLabel.setText("Vetna forma:");
    formLabel.setBounds(new Rectangle(20, 100, 130, 30));


    buttonPanel.add(applyButton, null);
    buttonPanel.add(backButton, null);
    buttonPanel.add(newButton, null);
    buttonPanel.add(gramNewButton, null);
    buttonPanel.add(gramUpdateButton, null);

    this.getContentPane().add(stepsScroll, null);
    this.getContentPane().add(chBoxPanel, null);
    this.getContentPane().add(buttonPanel, null);
    this.getContentPane().add(formScroll, null);
    this.getContentPane().add(prodsBox, null);
    this.getContentPane().add(formLabel, null);
    this.getContentPane().add(prodLabel, null);

    formScroll.getViewport().add(formField, null);
    stepsScroll.getViewport().add(stepsList, null);

    makeMenu(); //Vytvori menu
    this.show();  //Zobrazi okno
  }

  /**
   * Akce provedena pri stisku tlacitka zpet. Vrati derivaci k vybranemu
   * kroku. Pokud zadny vybrany neni, vrati derivaci o jeden krok.
   */
  private void backButtonAction() {
    if(this.stepsList.getSelectedIndex()==-1){
      this.derivation.backOneStep();
    }
    else{
      this.derivation.backToStep(stepsList.getSelectedIndex());
    }
    this.setSteps();
  }

  /**
   * Vytvori novou derivaci. Vsechny kroky aktualni derivace budou zapomenuty
   */
  private void newButtonAction() {
    this.derivation=new Derivation(this.grammar);
    this.backButton.setEnabled(false);
    this.menuItems[DERIVATION_MENU][BACK_ITEM].setEnabled(false);
    setSteps();
  }

  /**
   * Akce tlacitka nova gramatika. Otevre okno editace gramatiky s nevyplnenymi
   * pravidly.
   */
  private void gramNewButtonAction() {
    if(inputFrame!=null){
      inputFrame.dispose();
    }
    this.inputFrame = new PrgGramInputFrame(this, new ProgrammedGrammar());
  }

  /**
   * Akce pri volbe nacist gramatiku v menu. Otevre okne editace gramatiky a 
   * primo zvoli volbu otevreni souboru.
   */
  private void gramLoadButtonAction() {
    if(inputFrame!=null){
      inputFrame.dispose();
    }
    this.inputFrame = new PrgGramInputFrame(this, new ProgrammedGrammar());
    inputFrame.readFile();
  }

  /**
   * Akce pro tlacitko aplikuj. Provede dalsi krok derivace. Pokud je zvolena
   * automaticka derivace, zakaza pouzivani tlacitek mimo stop a nastavi
   * zpozdeni mezi kroky.
   */
  void applyButtonAction() {
    int index=this.prodsBox.getSelectedIndex(); //Zjisti vybrane pravidlo
    int appearance=0;
    Production p;
    try{                 //Zjisti pozadovany vyskyt leve strany
      appearance=(new Integer(this.appearField.getText())).intValue();
    }
    catch (NumberFormatException numFormE){ //Spatne vyplnene textove pole
      this.setEnabled(false);
      new ErrorFrame(this,ErrorFrame.WRONG_APPEAR_FORMAT);
      return;
    }
    if(index==-1){
      return;
    }
    try{
      if(index>=this.usableProds.length){ //Vybrano pravidlo v modu ac
        index-=this.usableProds.length;
        p=this.acUsableProds[index];
        derivation.applyAcProduction(p);
      }
      else{                   //Vybrano pravidlo pro klasickou aplikaci
        p=this.usableProds[index];
        derivation.applyProduction(p, appearance);
      }
    }
    catch (IndexOutOfBoundsException except){ //Pozadovany vyskyt neexistuje
      this.setEnabled(false);
      new ErrorFrame(this,ErrorFrame.TO_BIG_APPEAR);
      return;
    }
    this.backButton.setEnabled(true);  //Po aplikaci se jiz muze vracet 
                                       //v derivaci zpet
    this.menuItems[DERIVATION_MENU][BACK_ITEM].setEnabled(true);
    this.setSteps();                   //Nastavi kroky derivace do seznamu

    if(this.autoCHBox.isSelected()){  //Pokud je automaticka derivace
      this.stopped=false;            
      setAutoConf();           //Povoli jen tlacitko stop a ostatni zakaze
      this.setAlarm();         //Nastavi zpozdeni pred dalsi krok
    }
  }

  /**
   * Akce tlacitka upravit gramatiku. Otevre okne editace gramatiky a 
   * predvyplni pravidla a pocatecni neterminal.
   */
  void gramUpdateButtonAction() {
    if(inputFrame!=null){
      inputFrame.dispose();
    }
    this.inputFrame = new PrgGramInputFrame(this, grammar.copy());
  }

  /**
   * Akce tlacitka stop. Ukonci automatickou derivaci a povoli zase vsechna
   * tlacitka.
   */
  void stopButtonAction(){
    if(thread!=null){
      thread.interrupt();
      thread=null;
    }
    this.stopped=true;
    this.unsetAutoConf();
  }

  /**
   * Akce provadena pri zmene zaskrtnuti leftmost derivace. Pri zvoleni nastavi
   * vyskyt na 1 a zakaze jeho zmenu. Pri odskrtnuti povoli zmenu vyskytu. 
   * Navic znovu urci aplikovatelna pravidla, protoze i ona se zmenou leftmost 
   * omezeni mohla zmenit.
   */
  private void leftCHBoxStateChanged() {
    if(leftCHBox.isSelected()){
      this.appearField.setText("1");
      this.appearField.setEnabled(false);
    }
    else{
      if(!this.autoCHBox.isSelected()){
        this.appearField.setEnabled(true);
      }
    }
    this.setProds();
  }

  /**
   * Akce provedena pri zmene stavu tlacitka pro automatickou derivaci. Pri 
   * zvoleni umozni menit zpozdeni, povoli nahodnou derivaci, nastavi vysktyt
   * na 1 a zakaze jeho zmenu. Pri odskrtnuti se znemozni zmena zpozdeni,
   * odskrtne se nahodna volba a zakaze jeho zmena, povoli se editace vyskytu.
   */
  private void autoCHBoxStateChanged() {
    if(this.autoCHBox.isSelected()){
      this.delayField.setEnabled(true);
      this.randomCHBox.setEnabled(true);
      this.appearField.setText("1");
      this.appearField.setEnabled(false);
    }
    else{
      this.delayField.setEnabled(false);
      this.randomCHBox.setEnabled(false);
      this.randomCHBox.setSelected(false);
      if(!this.leftCHBox.isSelected()){
        this.appearField.setEnabled(true);
      }
    }
  }

  /**
   * Vytvori vechna menu vcetne polozek v nich. Vsem polozkam nastavi 
   * posluchace udalosti.
   */
  private void makeMenu(){
    JMenuBar menuBar=new JMenuBar();
    int i,j;
    ActionListener actListener = new ActionListener(){ //Posluchac udalosti
                                                      //od vsech polozek menu
      public void actionPerformed(ActionEvent ae){
        if(ae.getActionCommand()==menuItemsStrings[GRAMMAR_MENU][NEW_ITEM]){
          gramNewButtonAction();    //nova gramatika
        }
        if(ae.getActionCommand()==menuItemsStrings[GRAMMAR_MENU][LOAD_ITEM]){
          gramLoadButtonAction();   //nacteni gramatiky
        }
        if(ae.getActionCommand()==menuItemsStrings[GRAMMAR_MENU][CHANGE_ITEM]){
          gramUpdateButtonAction(); //editace gramatiky
        }
        if(ae.getActionCommand()==menuItemsStrings[GRAMMAR_MENU][SAVE_ITEM]){
          writeFile();              //ulozeni gramatiky do souboru
        }
        if(ae.getActionCommand()==menuItemsStrings[DERIVATION_MENU][NEW_ITEM]){
          newButtonAction();        //nova derivace
        }
        if(ae.getActionCommand()==menuItemsStrings[DERIVATION_MENU][BACK_ITEM]){
          backButtonAction();       //navrat v derivaci
        }
        if(ae.getActionCommand()==
                menuItemsStrings[DERIVATION_MENU][APPLY_ITEM]){
          applyButtonAction();     //aplikace pravidla
        }
        if(ae.getActionCommand()=="Stop"){
          stopButtonAction();     //aplikace pravidla
        }
        if(ae.getActionCommand()==menuItemsStrings[HELP_MENU][DER_ITEM]){
          if(hf==null){          //Napoveda k derivaci
            hf= new HelpFrame(HelpFrame.DERIVATION,
                                      HelpFrame.DERIVATION);
          }
          else {                //Napoveda jiz existuje, tak nespousti dalsi
            hf.show();
            hf.setPage(HelpFrame.DERIVATION); //Prepne na napovedu o derivaci
          }
        }
        if(ae.getActionCommand()==menuItemsStrings[HELP_MENU][ABOUT_ITEM]){
          if(hf==null){        //Napoveda o programu
            hf= new HelpFrame(HelpFrame.DERIVATION, HelpFrame.ABOUT);
          }
          else {
            hf.show();         //Napoveda jiz existuje, tak nespousti dalsi
            hf.setPage(HelpFrame.ABOUT); //Prepne na napovedu o programu
          }
        }
        if(ae.getActionCommand()==menuItemsStrings[DERIVATION_MENU][END_ITEM]){
          dispose();    //Zavreni okna
          try{
            System.exit(0);
          }
          catch(java.security.AccessControlException except){
          }
        }
      }
    };

    menuItems = new JMenuItem[menuStrings.length][];
    for(i=0;i<menuStrings.length;i++){  //Pres vsechna menu
      JMenu menu=new JMenu(menuStrings[i]);
      menuBar.add(menu);                //Prida menu do listy
      menuItems[i]=new JMenuItem[menuItemsStrings[i].length];
      for(j=0;j<menuItemsStrings[i].length;j++){  //Vsechny polozky v menu
        menuItems[i][j]=new JMenuItem(menuItemsStrings[i][j]);
        menuItems[i][j].addActionListener(actListener);
        menu.add(menuItems[i][j]);  //Prida polozku do menu
      }
    }
    this.setJMenuBar(menuBar);  //Nastavi listu
    //Nektere polozky na zacatku nejsou dovolene
    menuItems[DERIVATION_MENU][BACK_ITEM].setEnabled(false); 
    menuItems[DERIVATION_MENU][NEW_ITEM].setEnabled(false);
    menuItems[DERIVATION_MENU][APPLY_ITEM].setEnabled(false);
    menuItems[GRAMMAR_MENU][CHANGE_ITEM].setEnabled(false);
    menuItems[GRAMMAR_MENU][SAVE_ITEM].setEnabled(false);
    if(this.applet!=null){
      menuItems[GRAMMAR_MENU][LOAD_ITEM].setEnabled(false);
    }
  }

  /**
   * Vytvori nove vlakno, ktere pocka cas zadany v poli zpozdeni a potom 
   * vola metodu autoDerivate(). Nejprve ale kontroluje spravnost zadaneho 
   * zpozdeni.
   */
  private void setAlarm(){
    int del;
    try{
      del=(new Integer(this.delayField.getText())).intValue();
    }
    catch (NumberFormatException numFormE){ //Spatny format cisla v poli
      this.setEnabled(false);
      new ErrorFrame(this,ErrorFrame.WRONG_DELAY_FORMAT);
      this.unsetAutoConf();
      return;
    }
    this.delay=del;
    if(delay<0){  //Zpozdeni nesmi byt zaporne
      this.setEnabled(false);
      new ErrorFrame(this,ErrorFrame.WRONG_DELAY_FORMAT);
      this.unsetAutoConf();
      return;
    }
    if((this.usableProds.length + this.acUsableProds.length)==1 ||
        ((this.usableProds.length + this.acUsableProds.length)>0
           && this.randomCHBox.isSelected())){
      if(delay>=0){
        thread = new Thread(new Runnable(){  //Vytvori nove vlakno
          public void run(){
            if(delay>0){                     
              try{
                Thread.sleep(delay);         //Pocka urcenou dobu
              }
              catch(InterruptedException e){
              }
            }
            autoDerivate(); //Vola funkci pro dalsi krok automaticke derivace
          }
        });
        thread.start();    //Spusti nove vlakno
      }
    }
    else{   //Je aplikovatelnych vice pravidel a neni zvolena automaticka 
            //derivace, nebo neni zadne pouzitelne pravidlo
      this.unsetAutoConf();
      return;
    }
  }

  /**
   * Pokud nebyla automaticka derivace prerusena provede jeji dalsi krok. Pokud
   * je jeste mozne v derivaci pokracovat, nastavi zase metodou setAlarm() 
   * paralelni vlakno, ktere spusti dalsi krok.
   */
  private void autoDerivate(){
    if(stopped){ //Automatick derivace byla prerusena, tak nic nedelej
      return;
    }
    if((this.usableProds.length + this.acUsableProds.length)==1){
                  //Prave jedno aplikovatelne pravidlo
      if(this.usableProds.length==1){  //Klasicka aplikace
        derivation.applyProduction(usableProds[0]);
      }
      else {             //Aplikace ve smyslu testovani vyskytu
        derivation.applyAcProduction(acUsableProds[0]);
      }
      this.setSteps();   //Nastavi seznam kroku derivace
    }
    else if((this.usableProds.length + this.acUsableProds.length)>0
           && this.randomCHBox.isSelected()){
              //Nahodna volba z vice aplikovatelnych pravidel
      int prodCount=this.usableProds.length + this.acUsableProds.length;
      int randNum=(new Random()).nextInt(prodCount);  //Nahodne cislo
      if(randNum>=this.usableProds.length){ //Je v rozsahu pravidel pro ac 
                                            //aplikaci
        derivation.applyAcProduction(acUsableProds[randNum-usableProds.length]);
      }
      else{          //Je v rozsahu pravidel pro klasickou aplikaci
        derivation.applyProduction(usableProds[randNum]);
      }
      this.setSteps();  //Nastavi seznam kroku derivace
    }
    else{  //Neni mozne provest derivaci automaticky
      this.unsetAutoConf();
      return;
    }
    if(this.autoCHBox.isSelected()){   
      this.setAlarm(); //Nastavi znovu zpozdeni pred dalsim krokem
    }
  }

  /**
   * Pri automaticke derivaci je mozne stisknout jen tlacitko stop, ktere 
   * derivaci zastavi. Ostatni tlacitka a prvky v okne zakaze tato funkce.
   * Navic tlacitko stop vznikne prejmenovanim tlacitka aplikuj.
   */
  void setAutoConf(){
    this.applyButton.setText("Stop");
    this.menuItems[DERIVATION_MENU][APPLY_ITEM].setText("Stop");
    this.backButton.setEnabled(false);
    this.menuItems[DERIVATION_MENU][BACK_ITEM].setEnabled(false);
    this.gramUpdateButton.setEnabled(false);
    this.menuItems[GRAMMAR_MENU][CHANGE_ITEM].setEnabled(false);
    this.menuItems[GRAMMAR_MENU][SAVE_ITEM].setEnabled(false);
    this.newButton.setEnabled(false);
    this.menuItems[DERIVATION_MENU][NEW_ITEM].setEnabled(false);
    this.gramNewButton.setEnabled(false);
    this.menuItems[GRAMMAR_MENU][NEW_ITEM].setEnabled(false);
    this.menuItems[GRAMMAR_MENU][LOAD_ITEM].setEnabled(false);
    this.leftCHBox.setEnabled(false);
    this.autoCHBox.setEnabled(false);
    this.randomCHBox.setEnabled(false);
    this.delayField.setEnabled(false);
    this.appearField.setEnabled(false);
  }

  /**
   * Po skonceni automaticke derivace povoli zpet prvky rozhrani a zmeni
   * tlacitko stop na aplikuj.
   */
  void unsetAutoConf(){
    if((this.usableProds.length + this.acUsableProds.length)==0 ||
      derivation.isTerminalWord()){
      this.applyButton.setEnabled(false); //Neni mozne pokracovat v derivaci
      this.menuItems[DERIVATION_MENU][APPLY_ITEM].setEnabled(false);
    }
    this.applyButton.setText("Aplikuj");
    this.menuItems[DERIVATION_MENU][APPLY_ITEM].setText(
      menuItemsStrings[DERIVATION_MENU][APPLY_ITEM]);
    this.gramUpdateButton.setEnabled(true);
    this.menuItems[GRAMMAR_MENU][CHANGE_ITEM].setEnabled(true);
    this.menuItems[GRAMMAR_MENU][SAVE_ITEM].setEnabled(true);
    this.newButton.setEnabled(true);
    this.menuItems[DERIVATION_MENU][NEW_ITEM].setEnabled(true);
    this.gramNewButton.setEnabled(true);
    this.menuItems[GRAMMAR_MENU][NEW_ITEM].setEnabled(true);
    this.menuItems[GRAMMAR_MENU][LOAD_ITEM].setEnabled(true);
    this.autoCHBox.setEnabled(true);
    this.leftCHBox.setEnabled(true);
    autoCHBoxStateChanged();
    this.delayField.setEnabled(true);
    if(this.derivation.stepsCount() > 0){  
      this.backButton.setEnabled(true);  //Je se kam vracet v derivaci
      this.menuItems[DERIVATION_MENU][BACK_ITEM].setEnabled(true);
    }
  }

  /**
   * Slouzi k ulozeni gramatiky, podle ktere probiha derivace, do souboru.
   * Otevre dialog pro vyber souboru a zapise do nej gramatiku. Pokud soubor
   * existuje, bude prepsan bez varovani.
   */
  protected void writeFile(){
    JFileChooser chooser;
    if(directory!=null){
      chooser = new JFileChooser(directory);  //Nastavi posledni adresar, ze
    }                                     //ktereho byl vybran nejaky soubor
    else {
      chooser = new JFileChooser();
    }
    chooser.setDialogTitle("Vyberte soubor pro ulozeni gramatiky");
    chooser.setApproveButtonToolTipText("Ulozi gramatiku do vybraneho souboru");
    chooser.setDialogType(JFileChooser.SAVE_DIALOG);
    chooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
    int returnVal=chooser.showSaveDialog(this); //Vyber souboru
    if(returnVal==JFileChooser.APPROVE_OPTION){  //Byl vybran soubor
      directory=chooser.getSelectedFile().getParentFile(); //Zapamatuje si 
                                      //adresar pro dalsi vyber souboru
      PrgGrammarFile pgf = new PrgGrammarFile(chooser.getSelectedFile());
      try{
        pgf.writeGrammar(this.grammar);          //Zapsani gramatiky
      }
      catch(java.io.IOException e){
        this.setEnabled(false);
        new ErrorFrame(this,ErrorFrame.IO_ERROR);
      }
    }
  }
}
