package gui;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

/**
 * Okno pro zobrazeni chybovych hlaseni. Obsahuje jen text chyby a tlacitko
 * zavirajici okno.
 *
 * @author Martin Kot
 * @version 1.0
 */
public class ErrorFrame extends JFrame {
  /**Vyska okna*/
  public static final int WINDOW_HEIGHT = 140;
  /**Sirka okna*/
  public static final int WINDOW_WIDTH = 500;
  /**Cislo zobrazene zpravy*/
  protected int message;
  /**Zobrazeni textu chyby*/
  JLabel errorLabel = new JLabel();
  /**Tlacitko pro zavreni okna*/
  JButton okButton = new JButton();
  /**Objekt, ktery ma byt informovan o zavreni chyboveho okna*/
  Errorable prevFrame;
 
  /**Cislo chyby - nevyplnene oznaceni pravidla*/
  public static final int NO_LABEL=1;
  /**Cislo chyby - nevyplnena leva strana pravidla*/
  public static final int NO_LEFT=2;
  /**Cislo chyby - oznaceni pravidla jiz bylo jednou pouzito*/
  public static final int EXIST_LABEL=3;
  /**Cislo chyby - chybny format oznaceni pravidla*/
  public static final int WRONG_LABEL=4;
  /**Cislo chyby - prilis symbolu na leve strane pravidla*/
  public static final int LONG_LEFT=5;
  /**Cislo chyby - spatny format neterminalu*/
  public static final int WRONG_FORMAT=6;
  /**Cislo chyby - spatny format symbolu*/
  public static final int WRONG_FORMAT_TN=7;
  /**Cislo chyby - nedefinovano pravidlo s oznaceni v nekterem poli*/
  public static final int NONDEF_LABEL=8;
  /**Cislo chyby - neni urcen pocatecni neterminal*/
  public static final int NO_START=9;
  /**Cislo chyby - prilis mnoho symbolu v poli pro pocatecni*/
  public static final int LONG_START=10;
  /**Cislo chyby - spatny format pocatecniho symbolu*/
  public static final int WRONG_FORMAT_START=11;
  /**Cislo chyby - spatny format cisla v textovem poli vyskytu*/
  public static final int WRONG_APPEAR_FORMAT=12;
  /**Cislo chyby - pozadovany vyskyt neexistuje */
  public static final int TO_BIG_APPEAR=13;
  /**Cislo chyby - spatny format cisla v textovem poli zpozdeni*/
  public static final int WRONG_DELAY_FORMAT=14;
  /**Cislo chyby - chybny format souboru*/
  public static final int PARSE_ERROR=15;
  /**Cislo chyby - chyba pri cteni souboru*/
  public static final int IO_ERROR=16;
  /**Cislo chyby - soubor nenalezen*/
  public static final int FILE_NOT_FOUND=17;

  /**Texty zprav. Poradi v poli odpovida cislum chyb.*/
  protected static final String[] messages={
    "",
    "Neni vyplneno oznaceni pravidla",
    "Neni vyplnena leva strana pravidla",
    "Oznaceni pravidla jiz bylo uzito",
    "Oznaceni pravidla ma spatny format",
    "Leva strana obsahuje vice nez 1 neterminal",
    "Neterminal musi zacinat velkym pismenem",
    "Neterminal musi zacinat velkym pismenem, terminal malym",
    "Nebylo definovano pravidlo s oznacenim: ",
    "Neni vyplnen pocatecni neterminal",
    "Pocatacni neterminal musi byt prave jeden",
    "Neterminal musi zacinat velkym pismenem",
    "Vyskyt symbolu musi byt prirozene cislo",
    "Leva strana se vyskytuje menekrat, nez je pozadovano",
    "Zpozdeni musi byt nezaporne cele cislo",
    "",
    "Chyba pri cteni souboru",
    "Soubor nenalezen"
    };

  /**
   * Pomocny konstruktor. Vytvori okne bez zpravy.
   */
  public ErrorFrame() {
    try {
      makeFrame();
    }
    catch(Exception e) {
      e.printStackTrace();
    }
    prevFrame=null;
    message=0;
    prevFrame=new PrgGramInputFrame();
  }

  /**
   * Vytvori chybove okno bez zpravy. Je urceno, ktery objekt se ma informovat 
   * volanim takeControl() o zavreni chyboveho okna.
   *
   * @param f Objekt implementujici rozhrani Errorable, ktery bude informovan
   * o zavreni chyboveho okna.
   */
  public ErrorFrame(Errorable f) {
    prevFrame=f;
    try {
      makeFrame();
    }
    catch(Exception e) {
      e.printStackTrace();
    }
    message=0;
  }

  /**
   * Vytvori chybove okno s urcenou zpravou. Je urceno, ktery objekt se ma 
   * informovat volanim takeControl() o zavreni chyboveho okna.
   *
   * @param f Objekt implementujici rozhrani Errorable, ktery bude informovan
   * o zavreni chyboveho okna.
   * @param mes Cislo chyby, ktera ma byt zobrazena.
   */
  public ErrorFrame(Errorable f, int mes) {
    this(f);
    this.setMessage(mes);
  }

  /**
   * Nastavi konretni text zpravy. Umozni texty, ktere nejsou v seznamu.
   *
   * @param text Zobrazeny text chyby
   */
  public void setText(String text){
    this.errorLabel.setText(text);
  }

  /**
   * Nastavi cislo chyboveho textu a tim i tento text.
   *
   * @param m Cislo chyby
   */
  public void setMessage(int m){
    this.message=m;
    this.errorLabel.setText(this.messages[m]);
    this.okButton.requestFocus();
    this.show();
  }

  /**
   * Nastavi cislo chyby i text. Cislo tedy neni pouzito k vyberu textu ze
   * seznamu.
   *
   * @param m Cislo chyby
   * @param s Text chyboveho hlaseni
   */
  public void setMessage(int m, String s){
    this.message=m;
    this.errorLabel.setText((
      new StringBuffer(this.messages[m])).append(s).toString());
    this.okButton.requestFocus();
    this.show();
  }

  /**
   * Vytvori vsechny prvky okna, nastavi chybovy text, vytvori tlacitku 
   * posluchace udalosti.
   */
  private void makeFrame() {
    Dimension dim = Toolkit.getDefaultToolkit().getScreenSize();
    this.setSize(WINDOW_WIDTH,WINDOW_HEIGHT);
    this.setLocation((dim.width - WINDOW_WIDTH)/2,
      (dim.height - WINDOW_HEIGHT)/2); //Vycentrovani okna doprostred obrazovky

    this.addWindowListener(new WindowAdapter(){ //Posluchac udalosti okna
      public void windowClosing(WindowEvent e){  //Zavreni okna
        okButtonAction();
      }
    });

    this.addKeyListener(new KeyListener(){  //Posluchac udalosti z klavesnice
      public void keyPressed(KeyEvent e){
        if(e.getKeyCode()==KeyEvent.VK_ENTER){//Enter je jako stisk tlacitka OK
          okButtonAction();
        }
      }
      public void keyTyped(KeyEvent e){
      }
      public void keyReleased(KeyEvent e){
      }
    });

    this.getContentPane().setLayout(null);
    this.setTitle("Chyba");
    okButton.setText("OK");
    okButton.setToolTipText(
      "Zavre okno s chybovym hlasenim");
    okButton.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) { //Stisk tlacitka OK
        okButtonAction();
      }
    });
    errorLabel.setForeground(Color.red);
    this.getContentPane().add(errorLabel);
    this.getContentPane().add(okButton);

    this.errorLabel.setBounds(10,20,480,25);
    this.errorLabel.setAlignmentX(JLabel.CENTER);
    this.okButton.setBounds(210,50,80,25);
  }
 
  /**
   * Akce pro zavreni okna. Informuje o zavreni registrovany objekt typu 
   * Errorable.
   */
  void okButtonAction() {  
    this.dispose();                          //Zrusi okno
    this.prevFrame.takeControl(message);     //Informuje prislusny objekt
  }
}
