package gui;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
 * Okno pro zobrazeni napovedy
 *
 * @author Martin Kot
 * @version 1.00
 */
public class HelpFrame extends JFrame {
    /** Panel pro zobrazovani textovych poli s napovedou */
    private JPanel panelCard;
    /** Layout pro prepinani mezi dvemi kartami napovedy ("O aplikaci" a
     *  "Derivace" nebo "Gramatika")
     */
    private CardLayout cardLayout;

    /** Symbolicka konstanta pro oznaceni stranky napovedy o derivaci*/
    public static final int DERIVATION = 1;
    /** Symbolicka konstanta pro oznaceni stranky napovedy o gramatice*/
    public static final int GRAMMAR = 0;
    /** Symbolicka konstanta pro oznaceni stranky o programu */
    public static final int ABOUT = 2;

    /** Vyska okna napovedy */
    protected static int FRAME_HEIGHT = 420;
    /** Sirka okna napovedy */
    protected static int FRAME_WIDTH = 440;

    /** Typ okna - muze byt DERIVATION nebo GRAMMAR*/
    protected int type = DERIVATION;
    /** Text o programu */
    protected static String aboutText =
        "GrammarSimulator - Simulator programovane gramatiky\n\n" +
        "Autor: Martin Kot\n\n" +
        "Verze: 1.00\n" +
        "Datum: 25.4.2002\n\n\n" +
        "Program byl vytvoren jako soucast diplomove prace na tema \n"+
        "\"Rizene gramatiky\".\n\n\n"+
        "Program slouzi pro simulaci derivace podle programovane \n"+
        "gramatiky. Po startu se objevi okno \"Simulace derivace\", \n"+
        "ve kterem se ridi samotna derivace. Predem je ale nutno zadat\n"+
        "gramatiku. K tomu slouzi okno \"Editace programovane gramatiky\".\n"+
        "Popis ovladani je uveden v napovedach k temto dvema oknum \n"+
        "(v menu \"Napoveda\" polozka \"Derivace\" nebo \n"+
        "\"Editace gramatiky\".)";
    /** Text o editaci gramatiky*/
    protected static String grammarText =
        "Editace gramatiky \n\n"+
        "V horni casti okna je 5 textovych poli. Do nich se vypisuje\n"+
        "pravidlo.\n"+
        "Ozn. - Oznaceni pravidla. Jde o retezec znaku jednoznacne\n"+
        "    identifikujici pravidlo. Nesmi se pouzit vicekrat stejne.\n"+
        "Pravidlo - Pravidlo je rozdeleno na dve pole oddelena \"->\".\n"+
        "    Do prvniho pole se vypisuje leva strana. Jde o prave jeden\n"+
        "    neterminal. Druhe pole slouzi k zapsani prave strany \n"+
        "    slozene z terminalu i neterminalu. Terminal je retezec\n"+
        "    zacinajici malym pismenem, neterminal zacina velkym \n"+
        "    pismenem. Nesmi obsahovat zavorky, bile znaky, strednik,\n"+
        "    carku, dvojtecku, pomlcku, rovnitko, mensi a vetsi. Tyto\n"+
        "    znaky jednotlive terminaly a neterminaly oddeluji.\n"+
        "Uspech - Pole uspechu pravidla. Jde o seznam oznaceni pravidel\n"+
        "    oddelenych stejnymi znaky jako symboly abecedy na prave\n"+
        "    strane pravidla. Jestli existuji pravidla s oznacenimi\n "+
        "    uvedenymi v tomto poli, se kontroluje az po zadani cele\n"+
        "    gramatiky.\n"+
        "Neuspech - Pole neuspechu pravidla. Plati pro nej stejne\n"+
        "    omezeni, jako pro pole uspechu.\n\n"+
        "Vyplnene pravidlo se prida do gramatiky stiskem klavesy \"Enter\",\n"+
        "pokud je kurzor v nekterem z textovych poli pravidla, volbou\n"+
        "\"Pridat\" v menu \"Pravidlo\" nebo stiskem \"Pridat\". Pred\n"+
        "zarazenim pravidla do gramatiky se zkontroluje, jestli je\n"+
        "je spravne zadano.\n"+
        "Pravidla gramatiky jsou uvedena v seznamu. Pokud je nektere\n"+
        "vybrano, muze se stiskem \"Smazat\" nebo stejne pojmenovanou\n"+
        "volbou v menu \"Pravidlo\" vymazat. Tlacitko \"Upravit\" nebo\n"+
        "odpovidajici volba v menu pravidlo ze seznamu take odstrani, \n"+
        "ale vyplni jim textova pole v horni casti okna. Po oprave se\n"+
        "pravidlo muze znovu pridat stejne jako nove.\n\n"+
        "Radky \"T\" a \"N\" obsahuji seznam terminalu a neterminalu\n"+
        "gramatiky. Vyplnuji se podle pridavanych pravidel a nelze je\n"+
        "menit. Textove pole oznacene \"Pocatecni\" obsahuje startovaci\n"+
        "neterminal. Vyplni se automaticky levou stranou prvniho \n"+
        "zadaneho pravidla, ale je mozne jej menit.\n\n"+
        "Tlacitko \"Storno\" a stejna volba v menu \"Gramatika\" slouzi\n"+
        "k ukonceni editace gramatiky bez ulozeni zmen. Vsechny \n"+
        "provedene zmeny budou bez upozorneni zruseny! Tlacitko \"OK\"  \n"+
        "gramatiku prekontroluje a nastavi pro derivaci. Gramatika nemusi \n"+
        "obsahovat zadne pravidlo, ale vzdy musi byt vyplnen pocatecni \n"+
        "neterminal.\n\n"+
        "Gramatiku je mozne ulozit do souboru vybranim \"Ulozit\" \n"+
        "v menu \"Gramatika\". Otevre se dialog pro vyber souboru a stisk \n"+
        "\"Save\" provede ulozeni. Gramatika pred ulozenim musi mit urcen\n"+
        "pocatecni neterminal a vsechna pravidla uvedena v polich \n"+
        "uspechu i neuspechu musi byt definovana.\n"+
        "Nacteni gramatiky ze souboru se provede po volbe \"Nacist\" \n"+
        "v menu \"Gramatika\" a stisku \"Open\" v otevrenem okne pro  \n"+
        "vyber souboru. Nactena gramatika prepise bez varovani prave\n"+
        "editovanou!\n";

    /** Text napovedy o derivaci */
    protected static String derivationText =
        "Simulace derivace \n\n"+
        "Derivace probiha podle programovane gramatiky.\n"+
        "Zadani gramatiky je mozne ze souboru po zvoleni \"Nacist\"\n"+
        "v menu \"Gramatika\". Otevre se dialog pro vybrani \n"+
        "konkretniho souboru a po potvrzeni tlacikem \"Open\" \n"+
        "je gramatika nactena a zobrazena v okne editace gramatiky. \n"+
        "Gramatiku je mozne take zadat z klavesnice po zvoleni \n"+
        "\"Nova\" v menu \"Gramatika\" nebo po stisku tlacitka \n"+
        "\"Nova gramatika\". \n\n"+
        "Byla-li gramatika jiz zadana, lze ji ulozit do souboru\n"+
        "pomoci \"Ulozit\" v menu \"Gramatika\". Gramatiku je mozne\n"+
        "kdykoliv upravit. Volbou \"Upravit\" v menu \"Gramatika\"\n"+
        "nebo tlacitka \"Upravit gramatiku\" se otevre okno editace.\n\n"+
        "Na zacatku derivace je vetna forma tvorena startovacim\n"+
        "neterminalem. V rozbalovacim seznamu \"Vybrat pravidlo\"\n"+
        "jsou aplikovatelna pravidla. Pokud jde o aplikaci ve smyslu\n"+
        "testovani vyskytu je pred nim \"ac:\". Pravidlo je\n"+
        "aplikovano volbou \"Aplikuj pravidlo\" v menu \"Derivace\"\n"+
        "nebo stiskem \"Aplikuj\".\n"+
        "Tlacitko \"Zpet\" nebo polozka \"Zpet\" v menu \"Derivace\"\n"+
        "vrati derivaci o krok zpet, pokud neni vybran zadny radek\n"+
        "v seznamu. Pokud krok je vybran, vrati se derivace az pred\n"+
        "nej. \"Nova\" v menu \"Derivace\" a tlacitko \"Nova derivace\"\n"+
        "vrati derivaci na zacatek.\n\n"+
        "Volby:\n"+
        "Automaticky - Po stisku aplikuj jsou automaticky pouzivana\n"+
        "    pravidla, dokud je prubeh derivace jednoznacny. Pokud\n"+
        "    se leva strana vyskytuje ve vetne forme vicekrat, je\n"+
        "    vzdy prepsan nejlevejsi vyskyt. Mezi jednotlivymi kroky\n"+
        "    se ceka tolik milisekund, kolik je uvedeno v poli\n"+
        "    \"Zpozdeni\". Ukoncit automaticky prubeh je mozne stiskem\n"+
        "    \"Stop\" nebo volbou \"Stop\" v menu \"Derivace\". Pokud\n"+
        "    je pouzitelnych pravidel vice, je automaticky prubeh\n"+
        "    prerusen. Po vyberu pravidla se pokracuje stiskem \n"+
        "    \"Aplikuj\". Automaticka derivace konci, kdyz je \n"+
        "    vygenerovano terminalni slovo nebo neni aplikovatelne\n"+
        "    zadne pravidlo\n"+
        "Leftmost - Ze vsech aplikovatelnych pravidel vybere jen ta,\n"+
        "    ktera maji levou stranu ve vetne forme nejvice vlevo.\n"+
        "Nahodne - Je mozno zvolit jen pri automaticke derivaci.\n"+
        "    Pokud neni pravidlo urceno jednoznacne, je vybrano \n"+
        "    nahodne jedno z aplikovatelnych.\n"+
        "Vyskyt - Pri automaticke nebo leftmost derivaci je nastaven\n"+
        "    na 1. Jinak jde o vyskyt leve strany, na ktery se ma\n"+
        "    pravidlo pouzit. Pokud je cislo prilis velke, zobrazi se\n"+
        "    chybove hlaseni.\n"+
        "Zpozdeni - Je mozne nastavit jen pri zvolene automaticke\n"+
        "    derivaci. Jedna se o cas v milisekundach mezi pouzitim\n"+
        "    dvou pravidel." ;

    /**
     * Vytvori okno a zobrazi zvolenou stranku napovedy
     *
     * @param page Stranka, ktera bude po vytvoreni napovedy zobrazena jako
     * prvni
     * @param type Typ napovedy. Muze byt DERIVATION nebo GRAMMAR
     *
     */
    public HelpFrame(int type, int page) {
        //Vytvori nove okno s nadpisem
        if(type==DERIVATION){
          this.setTitle("Napoveda - derivace");
        }
        else {
          this.setTitle("Napoveda - editace gramatiky");
        }
        this.type=type;
       Dimension dim = Toolkit.getDefaultToolkit().getScreenSize();
       this.setLocation((dim.width - FRAME_WIDTH)/2,
         (dim.height - FRAME_HEIGHT)/2);

        //Panel s tlacitky na meneni stranek
        JPanel buttonPanel;
        //Tlacitko pro stranku s napovedou o derivaci nebo gramatice
        JButton buttonHelp;
        //Tlacitko pro informace o programu
        JButton buttonAbout;
        //Panel s napovedou o derivaci nebo o editaci gramatiky
        JPanel panelHelp;
        //Panel s textem o programu
        JPanel panelAbout;
        //Textove pole s textem o programu
        JTextArea textAbout;
        //Textove pole s textem napovedy o derivaci nebo o editaci gramatiky
        JScrollPane aboutScroll = new JScrollPane();
        JTextArea textHelp;
        //Posuvne listy k napovede
        JScrollPane derivationScroll = new JScrollPane();
        //Posluchac udalosti od tlacitek
        ActionListener actListener;
        JPanel pane = (JPanel)this.getContentPane();

        //Nastavi vlastnosti okna
        pane.setLayout(new BorderLayout(10,10));
        setSize(FRAME_WIDTH,FRAME_HEIGHT);

        panelCard = new JPanel();

        //Vytvoreni panelu s tlacitky na zmenu stranek
        buttonPanel = new JPanel();
        buttonPanel.setLayout(new FlowLayout());
        if(type==DERIVATION){
          buttonHelp = new JButton("Derivace");
          buttonHelp.setToolTipText(
            "Prepne na stranku s napovedou o derivaci");
        }
        else{
          buttonHelp = new JButton("Editace gramatiky");
          buttonHelp.setToolTipText(
            "Prepne na stranku s napovedou o editaci gramatiky");
        }
        buttonAbout = new JButton("O programu");
        buttonAbout.setToolTipText(
            "Prepne na stranku s informaci o programu");
        buttonPanel.add(buttonHelp);
        buttonPanel.add(buttonAbout);

        //Vytvoreni panelu o programu
        panelAbout = new JPanel();
        panelAbout.setLayout(new GridLayout(1,1,0,0));
        textAbout = new JTextArea();
        textAbout.setText(aboutText);
        textAbout.setEditable(false);
        aboutScroll.getViewport().add(textAbout);
        aboutScroll.setHorizontalScrollBarPolicy(
          JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
        aboutScroll.setVerticalScrollBarPolicy(
          JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
        panelAbout.add(aboutScroll);

        //Vytvoreni panelu napovedy
        panelHelp = new JPanel();
        panelHelp.setLayout(new GridLayout(1,1,0,0));
        textHelp = new JTextArea();
        if(type==DERIVATION){
          textHelp.setText(derivationText);
        }
        else{
          textHelp.setText(grammarText);
        }
        textHelp.setEditable(false);
        derivationScroll.getViewport().add(textHelp);
        derivationScroll.setHorizontalScrollBarPolicy(
          JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
        derivationScroll.setVerticalScrollBarPolicy(
          JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
        panelHelp.add(derivationScroll);

        //Vytvoreni layoutu na prepinani mezi strankami napovedy
        cardLayout = new CardLayout();
        panelCard.setLayout(cardLayout);
        if(type==DERIVATION){
          panelCard.add(panelHelp,"Derivace");
        }
        else {
          panelCard.add(panelHelp,"Editace gramatiky");
        }
        panelCard.add(panelAbout,"O programu");
        if (page == DERIVATION || page == GRAMMAR) {
            cardLayout.first(panelCard);
            buttonHelp.requestFocus();
        }
        else {
            buttonAbout.requestFocus();
            cardLayout.last(panelCard);
        }

        //Pridani posluchace na udalosti od tlacitek na meneni stranek
        actListener = new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (e.getActionCommand() == "Derivace" ||
                    e.getActionCommand() == "Editace gramatiky") {
                    cardLayout.first(panelCard);
                }
                else if (e.getActionCommand() == "O programu") {
                    cardLayout.last(panelCard);
                }
            }
        };
        buttonHelp.addActionListener(actListener);
        buttonAbout.addActionListener(actListener);
        pane.add("North",buttonPanel);
        pane.add("Center",panelCard);

        //Vytvoreni posluchace na obsluhu uzavreni okna
        addWindowListener(
            new WindowAdapter() {
                public void windowClosing(WindowEvent e) {
                    dispose();
                 }
            });

        show();
    }

    /**
     * Vytvori okno napovedy s implicitne nastavenou strankou s napovedou 
     * o derivaci.
     *
     */
    public HelpFrame() {
        this(DERIVATION, DERIVATION);
    }

    /**
     * Slouzi k nastaveni stranky napovedy
     *
     * @param page Pozadovana stranka napovedy
     */
    public void setPage(int page){
      if (page == DERIVATION || page==GRAMMAR) {
        cardLayout.first(panelCard);
      }
      else {
        cardLayout.last(panelCard);
      }
    }
}
