package gui;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import grammars.*;
import java.util.*;

/**
 * Okno pro zadavani programovane gramatiky. Umoznuje pridavat, menit a mazat
 * pravidla a urcit pocatecni neterminal. Gramatiku je mozne ulozit do souboru
 * i ze souboru nacist. 
 *
 * @author Martin Kot
 * @version 1.0
 */
public class PrgGramInputFrame extends JFrame implements Errorable{
  /**Okno zobrazujici chybova hlaseni*/
  protected ErrorFrame errorFrame;
  /**Okno derivace, ze ktereho bylo toto otevreno*/
  protected DerivationFrame derivationFrame;
  /**Okno napovedy*/
  protected HelpFrame hf=null;
  /**Editovana gramatika*/
  protected ProgrammedGrammar grammar;

  /**Hlavni panel okna*/
  private JPanel contentPanel;
  /**Panel pro tlacitka*/
  private JPanel buttonPanel=new JPanel();
  /**Nadpis k poli pro oznaceni pravidla*/
  private JLabel labelLabel = new JLabel();
  /**Nadpis k poli se startovacim neterminalem*/
  private JLabel startLabel = new JLabel();
  /**Nadpis k polim pravidla*/
  private JLabel prodLabel = new JLabel();
  /**Nadpis k poli s terminaly*/
  private JLabel termLabel = new JLabel();
  /**Nadpis k poli s neterminaly*/
  private JLabel nontermLabel = new JLabel();
  /**Nadpis k poli s polem uspechu*/
  private JLabel failLabel = new JLabel();
  /**Nadpis k poli s polem neuspechu*/
  private JLabel succLabel = new JLabel();
  /**Sipka oddelujici levou a pravou stranu pravidla*/
  private JLabel arrowLabel = new JLabel();
  /**Tlacitko pro ulozeni gramatiky a zavreni okna*/
  private JButton okButton = new JButton();
  /**Tlacitko pro zavreni okna bez ulozeni zmen*/
  private JButton stornoButton = new JButton();
  /**Tlacitko pro zmenu pravidla*/
  private JButton updateButton = new JButton();
  /**Tlacitko pro pridani pravidla*/
  private JButton addButton = new JButton();
  /**Tlacitko pro smazani pravidla*/
  private JButton deleteButton = new JButton();
  /**Textove pole pro oznaceni pravidla*/
  private JTextField labelField = new JTextField();
  /**Textove pole pro levou stranu pravidla*/
  private JTextField leftField = new JTextField();
  /**Textove pole pro pravou stranu pravidla*/
  private JTextField rightField = new JTextField();
  /**Textove pole pro oznaceni pravidel pole uspechu*/
  private JTextField succField = new JTextField();
  /**Textove pole pro oznaceni pravidel pole neuspechu*/
  private JTextField failField = new JTextField();
  /**Textove pole pro pocatecni neterminal*/
  private JTextField startField = new JTextField();
  /**Posuvne listy pro textove pole terminalu*/
  private JScrollPane termScroll;
  /**Posuvne listy pro textove pole neterminalu*/
  private JScrollPane nontermScroll;
  /**Posuvne listy pro seznam pravidel*/
  private JScrollPane listScroll;
  /**Textove pole pro zobrazeni terminalu*/
  private JTextField termField = new JTextField();
  /**Textove pole pro zobrazeni neterminalu*/
  private JTextField nontermField = new JTextField();
  /**Layout pro serazeni tlacitek do sloupce*/
  private GridLayout buttonsLayout = new GridLayout();
  /**Posluchac udalosti z klavesnice*/
  private ProdFieldKeyListener prodKeyListener = new ProdFieldKeyListener();
  /**Seznam pravidel*/
  private JList prodList = new JList();

  /**Vyska okna*/
  public static final int WINDOW_HEIGHT = 500;
  /**Sirka okna*/
  public static final int WINDOW_WIDTH = 600;
  /**Y souradnice textovych poli pravidla*/
  private static final int FIELD_Y = 60;
  /**Vyska textovych poli pravidla*/
  private static final int FIELD_HEIGHT = 25;
  /**Y souradnice nadpisu textovych poli pravidla*/
  private static final int LABS_Y = 30;
  /**Vyska nadpisu textovych poli pravidla*/
  private static final int LABS_HEIGHT = 25;
  /**Vyska seznamu pravidel*/
  private static final int LIST_HEIGHT = 200;
  /**Sirka seznamu pravidel*/
  private static final int LIST_WIDTH = 440;
  /**Vyska tlacitek*/
  private static final int BUTTONS_HEIGHT = 25;
  /**X souradnice tlacitek*/
  private static final int BUTTONS_X = 40+LIST_WIDTH;
  /**Sirka tlacitek*/
  private static final int BUTTONS_WIDTH = 100;
  /**X souradnice textoveho pole oznaceni pravidla*/
  private static final int LABEL_X = 20;
  /**Sirka textoveho pole oznaceni pravidla*/
  private static final int LABEL_WIDTH = 50;
  /**X souradnice textoveho pole leve strany pravidla*/
  private static final int LEFT_X = LABEL_X + LABEL_WIDTH + 30;
  /**Sirka textoveho pole leve strany pravidla*/
  private static final int LEFT_WIDTH = 50;
  /**X souradnice textoveho pole prave strany pravidla*/
  private static final int RIGHT_X = LEFT_X + LEFT_WIDTH + 30;
  /**Sirka textoveho pole prave strany pravidla*/
  private static final int RIGHT_WIDTH = 140;
  /**X souradnice textoveho pole pro pole uspechu pravidla*/
  private static final int SUCC_X = RIGHT_X + RIGHT_WIDTH + 30;
  /**Sirka textoveho pole pro pole uspechu pravidla*/
  private static final int SUCC_WIDTH = 100;
  /**X souradnice textoveho pole pro pole neuspechu pravidla*/
  private static final int FAIL_X = SUCC_X + SUCC_WIDTH + 30;
  /**Sirka textoveho pole pro pole neuspechu pravidla*/
  private static final int FAIL_WIDTH = 100;
  /**Y souradnice textoveho pole pro terminaly*/
  private static final int TERM_Y = 105;
  /**X souradnice textoveho pole pro terminaly*/
  private static final int TERM_X = 105;
  /**Vyska textoveho pole pro terminaly*/
  private static final int TEXTAREA_HIGHT=40;
  /**Y souradnice textoveho pole pro neterminaly*/
  private static final int NONTERM_Y=TERM_Y+TEXTAREA_HIGHT+20;
  /**Sirka seznamu pravidel*/
  private static final int LIST_Y = NONTERM_Y+TEXTAREA_HIGHT+20;
  /**Sirka nadpisu k textovym polim s neterminaly a terminaly*/
  private static final int TERM_LAB_WIDTH = 30;

  /**Nadpisu menu*/
  private static final String menus[]={"Gramatika","Pravidlo","Napoveda"};
  /**Nadpisy polozek jednotlivych polozek v menu*/
  private static final String menuItemStrings[][]=
    {{"Nacist","Ulozit","Zrusit","OK"},
     {"Pridat","Upravit","Smazat"},
     {"Editace gramatiky","O programu"}};
  /**Poradi menu gramatika*/
  private static final int FILE_MENU = 0;
  /**Poradi menu pravidlo*/
  private static final int PROD_MENU = 1;
  /**Poradi menu napoveda*/
  private static final int HELP_MENU = 2;
  /**Poradi polozky nacist ze souboru*/
  private static final int OPEN_ITEM = 0;
  /**Poradi polozky ulozeni do souboru*/
  private static final int SAVE_ITEM = 1;
  /**Poradi polozky zavreni okna bez ulozeni zmen*/
  private static final int STORNO_ITEM = 2;
  /**Poradi polozky ukonceni editace s ulozenim zmen*/
  private static final int OK_ITEM = 3;
  /**Poradi polozky pridat pravidlo*/
  private static final int ADD_ITEM = 0;
  /**Poradi polozky zmenit pravidlo*/
  private static final int CHANGE_ITEM = 1;
  /**Poradi polozky smazat pravidlo*/
  private static final int DEL_ITEM = 2;
  /**Poradi polozky napoveda o editaci gramatiky*/
  private static final int GRAM_ITEM = 0;
  /**Poradi polozky napoveda o programu*/
  private static final int ABOUT_ITEM = 1;


/**
 * Vnitrni trida slouzici jako posluchac udalosti z klavesnice. Po stisku
 * klavesy enter prida prave editovane pravidlo do gramatiky.
 */
protected class ProdFieldKeyListener implements KeyListener{
  public void keyPressed(KeyEvent e){
    if(e.getKeyCode()==KeyEvent.VK_ENTER){
      addButtonAction();
      labelField.requestFocus();
    }
  }
  public void keyTyped(KeyEvent e){
  }
  public void keyReleased(KeyEvent e){
  }
}

  /**
   * Pomocny konstruktor. Vytvori si novou gramatiku a nove okno derivace.
   */
  public PrgGramInputFrame() {
    this.grammar=new ProgrammedGrammar();
    errorFrame=new ErrorFrame(this);
    this.derivationFrame=new DerivationFrame();
    try {
      makeFrame();
    }
    catch(Exception e) {
      e.printStackTrace();
    }
  }

  /**
   * Vytvori okno pro editaci urcene gramatiky. Jeji hodnoty predvyplni do 
   * prvku okna.
   *
   * @param df Okno derivace, ze ktereho byla editace gramatiky volana
   * @param pg Editovana programovana gramatika
   */
  public PrgGramInputFrame(DerivationFrame df,Grammar pg) {
    this.grammar=(ProgrammedGrammar)pg;
    errorFrame=new ErrorFrame(this);
    this.derivationFrame=df;
    if(this.grammar.getStart()!=null){
      this.startField.setText(this.grammar.getStart().toString());
    }
    try {
      makeFrame();
    }
    catch(Exception e) {
      e.printStackTrace();
    }
    this.updateList();
    updateSymbols();
  }

  /**
   * Vytvori okno vcetne vsech jeho prvku. Priradi prvkum posluchace udalosti.
   */
  private void makeFrame() {
    Dimension dim = Toolkit.getDefaultToolkit().getScreenSize();
    this.setSize(WINDOW_WIDTH,WINDOW_HEIGHT);
    this.setLocation((dim.width - WINDOW_WIDTH)/2,
      (dim.height - WINDOW_HEIGHT)/2);  //Vycentruje okno doprostred obrazovky

    this.addWindowListener(new WindowAdapter(){
      public void windowClosing(WindowEvent e){
        stornoButtonAction();          //Zavreni okna je jako stisk storno
      }
    });
    this.contentPanel=(JPanel)this.getContentPane();
    this.setTitle("Editace programovane gramatiky");
    contentPanel.setLayout(null);

    buttonsLayout.setColumns(1);
    buttonsLayout.setRows(5);
    buttonsLayout.setVgap(10);
    buttonPanel.setEnabled(false);
    buttonPanel.setBounds(new Rectangle(BUTTONS_X, LIST_Y, 
                                        BUTTONS_WIDTH, LIST_HEIGHT));
    buttonPanel.setLayout(buttonsLayout);

    addButton.setText("Pridat");
    addButton.setToolTipText(
      "Prida pravidlo z hornich poli do mnoziny pravidel");
    addButton.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        addButtonAction();   //Akce pro pridani pravidla do gramatiky
      }
    });
    deleteButton.setText("Smazat");
    deleteButton.addActionListener(
      new java.awt.event.ActionListener() {
        public void actionPerformed(ActionEvent e) {
          deleteButtonAction();  //Smazani vybraneho pravidla
        }
      });
    deleteButton.setToolTipText(
      "Odstrani vybrane pravidlo z mnoziny pravidel");
    updateButton.setText("Upravit");
    updateButton.addActionListener(
      new java.awt.event.ActionListener() {
        public void actionPerformed(ActionEvent e) {
          updateButtonAction();  //Zmena vybraneho pravidla
        }
      });
    updateButton.setToolTipText(
      "Editace vybraneho pravidla");
    okButton.setText("OK");
    okButton.setToolTipText("Ukonci editaci a potvrdi zadanou gramatiku");
    okButton.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        okButtonAction();  //Potvrzeni editovane gramatiky
      }
    });
    stornoButton.setText("Zrusit");
    stornoButton.setToolTipText(
      "Zavre okno editace gramatiky bez ulozeni zmen");
    stornoButton.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        stornoButtonAction();  //Navrat k puvodni gramatice v okne derivace
      }
    });
    labelField.setBounds(new Rectangle(LABEL_X, FIELD_Y,
      LABEL_WIDTH, FIELD_HEIGHT));
    labelField.setToolTipText("Oznaceni pravidla");
    labelField.addKeyListener(prodKeyListener);
    leftField.setBounds(new Rectangle(LEFT_X, FIELD_Y,
      LEFT_WIDTH, FIELD_HEIGHT));
    leftField.setToolTipText("Leva strana pravidla");
    leftField.addKeyListener(prodKeyListener);
    rightField.setBounds(new Rectangle(RIGHT_X, FIELD_Y,
      RIGHT_WIDTH, FIELD_HEIGHT));
    rightField.setToolTipText("Prava strana pravidla");
    rightField.addKeyListener(prodKeyListener);
    succField.setBounds(new Rectangle(SUCC_X, FIELD_Y,
      SUCC_WIDTH, FIELD_HEIGHT));
    succField.setToolTipText("Oznaceni pravidel pole uspechu");
    succField.addKeyListener(prodKeyListener);
    failField.setBounds(new Rectangle(FAIL_X, FIELD_Y,
      FAIL_WIDTH, FIELD_HEIGHT));
    failField.setToolTipText("Oznaceni pravidel pole neuspechu");
    failField.addKeyListener(prodKeyListener);
    failField.setNextFocusableComponent(startField);
    startField.setBounds(new Rectangle(FAIL_X, NONTERM_Y,
      FAIL_WIDTH, FIELD_HEIGHT));
    startField.setToolTipText("Pocatecni neterminal");
    startField.setNextFocusableComponent(this.prodList);


    nontermField.setText("");
    nontermField.setToolTipText("Mnozina neterminalu obsazenych v pravidlech");
    nontermField.setColumns(nontermField.getText().length());
    nontermField.setEditable(false);
    nontermScroll = new JScrollPane(nontermField);
    nontermScroll.setHorizontalScrollBarPolicy(
      JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
    nontermScroll.setVerticalScrollBarPolicy(
      JScrollPane.VERTICAL_SCROLLBAR_NEVER);
    nontermScroll.setBounds(new Rectangle(LABEL_X+TERM_LAB_WIDTH,
      NONTERM_Y, LIST_WIDTH-TERM_LAB_WIDTH, TEXTAREA_HIGHT));

    termField.setText("");
    termField.setEditable(false);
    termField.setToolTipText("Mnozina terminalu obsazenych v pravidlech");
    termScroll = new JScrollPane(termField);
    termScroll.setHorizontalScrollBarPolicy(
      JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
    termScroll.setVerticalScrollBarPolicy(
      JScrollPane.VERTICAL_SCROLLBAR_NEVER);
    termScroll.setBounds(new Rectangle(LABEL_X+TERM_LAB_WIDTH,
      TERM_Y, LIST_WIDTH-TERM_LAB_WIDTH, TEXTAREA_HIGHT));

    arrowLabel.setText("->");
    arrowLabel.setBounds(new Rectangle(LEFT_X+LEFT_WIDTH,
      FIELD_Y, 30, FIELD_HEIGHT));
    arrowLabel.setAlignmentX(JLabel.CENTER_ALIGNMENT);

    startLabel.setText("Pocatecni");
    startLabel.setBounds(new Rectangle(FAIL_X,
      NONTERM_Y-FIELD_HEIGHT-5, FAIL_WIDTH, FIELD_HEIGHT));
    labelLabel.setText("Ozn.");
    labelLabel.setBounds(new Rectangle(LABEL_X, LABS_Y,
      LABEL_WIDTH, LABS_HEIGHT));
    prodLabel.setText("Pravidlo");
    prodLabel.setBounds(new Rectangle(LEFT_X, LABS_Y,
      LEFT_WIDTH+RIGHT_WIDTH, LABS_HEIGHT));
    succLabel.setText("Uspech");
    succLabel.setBounds(new Rectangle(SUCC_X, LABS_Y,
      SUCC_WIDTH, LABS_HEIGHT));
    failLabel.setText("Neuspech");
    failLabel.setBounds(new Rectangle(FAIL_X, LABS_Y,
      FAIL_WIDTH, LABS_HEIGHT));
    termLabel.setText("T");
    termLabel.setAlignmentX(JLabel.CENTER_ALIGNMENT);
    termLabel.setBounds(new Rectangle(LABEL_X, TERM_Y,
      TERM_LAB_WIDTH, LABS_HEIGHT));
    nontermLabel.setText("N");
    nontermLabel.setAlignmentX(JLabel.CENTER_ALIGNMENT);
    nontermLabel.setBounds(new Rectangle(LABEL_X, NONTERM_Y,
      TERM_LAB_WIDTH, LABS_HEIGHT));

    prodList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    prodList.setToolTipText("Seznam pravidel gramatiky");
    prodList.addKeyListener(new KeyListener(){
      public void keyPressed(KeyEvent e){
        if(e.getKeyCode()==KeyEvent.VK_DELETE){
          deleteButtonAction();  //Smazani vybraneho pravidla
        }
      }
      public void keyTyped(KeyEvent e){
      }
      public void keyReleased(KeyEvent e){
      }
    });
    listScroll = new JScrollPane(prodList);
    listScroll.setHorizontalScrollBarPolicy(
      JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
    listScroll.setVerticalScrollBarPolicy(
      JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
    listScroll.setBounds(new Rectangle(20, LIST_Y,
      LIST_WIDTH, LIST_HEIGHT));

    contentPanel.add(succField, null);
    contentPanel.add(labelLabel, null);
    contentPanel.add(arrowLabel, null);
    contentPanel.add(leftField, null);
    contentPanel.add(rightField, null);
    contentPanel.add(labelField, null);
    contentPanel.add(failField, null);
    contentPanel.add(prodLabel, null);
    contentPanel.add(succLabel, null);
    contentPanel.add(failLabel, null);
    contentPanel.add(startField, null);
    contentPanel.add(startLabel, null);
    contentPanel.add(termLabel, null);
    contentPanel.add(nontermLabel, null);
    buttonPanel.add(addButton, null);
    buttonPanel.add(updateButton, null);
    buttonPanel.add(deleteButton, null);
    buttonPanel.add(stornoButton, null);
    buttonPanel.add(okButton, null);
    contentPanel.add(buttonPanel,null);
    contentPanel.add(listScroll, null);
    contentPanel.add(termScroll);
    contentPanel.add(nontermScroll);
    makeMenu();
    this.show();
  }

  /**
   * Vytvori kompletni menu vcetne polozek. Jednotlivym polozkam priradi 
   * posluchac udalosti.
   */
  protected void makeMenu(){
    JMenuBar menuBar=new JMenuBar();
    int i,j;
    ActionListener actListener = new ActionListener(){ //Posluchac udalosti
      public void actionPerformed(ActionEvent ae){
        if(ae.getActionCommand()==menuItemStrings[FILE_MENU][OPEN_ITEM]){
          readFile(); //Nacist soubor
        }
        if(ae.getActionCommand()==menuItemStrings[FILE_MENU][SAVE_ITEM]){
          writeFile(); //Zapsat do souboru
        }
        if(ae.getActionCommand()==menuItemStrings[FILE_MENU][STORNO_ITEM]){
          stornoButtonAction();  //Zavreni okna bez ulozeni zmen
        }
        if(ae.getActionCommand()==menuItemStrings[FILE_MENU][OK_ITEM]){
          okButtonAction();  //Zavreni okna s ulozenim zmen
        }
        if(ae.getActionCommand()==menuItemStrings[PROD_MENU][ADD_ITEM]){
          addButtonAction();  //Pridani pravidla
        }
        if(ae.getActionCommand()==menuItemStrings[PROD_MENU][CHANGE_ITEM]){
          updateButtonAction(); //Editace vybraneho pravidla
        }
        if(ae.getActionCommand()==menuItemStrings[PROD_MENU][DEL_ITEM]){
          deleteButtonAction(); //Smazani vybraneho pravidla
        }
        if(ae.getActionCommand()==menuItemStrings[HELP_MENU][GRAM_ITEM]){
          if(hf==null){   //Zobrazeni noveho okna s napovedou
            hf= new HelpFrame(HelpFrame.GRAMMAR,
                                      HelpFrame.GRAMMAR);
          }
          else {         //Okno s napovedou jiz existuje
            hf.show();
            hf.setPage(HelpFrame.GRAMMAR); //Nastavi pozadovanou stranku
          }
        }
        if(ae.getActionCommand()==menuItemStrings[HELP_MENU][ABOUT_ITEM]){
          if(hf==null){  //Zobrazeni noveho okna s napovedou
            hf= new HelpFrame(HelpFrame.GRAMMAR, HelpFrame.ABOUT);
          }
          else {       //Okno s napovedou jiz existuje
            hf.show();
            hf.setPage(HelpFrame.ABOUT); //Nastavi pozadovanou stranku
          }
        }
      }
    };
    JMenuItem menuItems[][] = new JMenuItem[menus.length][];
    for(i=0;i<menus.length;i++){ //Pres vsechna menu
      JMenu menu=new JMenu(menus[i]);
      menuBar.add(menu);
      menuItems[i]=new JMenuItem[menuItemStrings[i].length];
      for(j=0;j<menuItemStrings[i].length;j++){ //Pres polozky v menu
        menuItems[i][j]=new JMenuItem(menuItemStrings[i][j]);
        menuItems[i][j].addActionListener(actListener);
        menu.add(menuItems[i][j]);
      }
    }
    this.setJMenuBar(menuBar);  //Nastaveni listy menu
    if(this.derivationFrame.isApplet()){ //Applet neumi pracovat se soubory
      menuItems[FILE_MENU][OPEN_ITEM].setEnabled(false);
      menuItems[FILE_MENU][SAVE_ITEM].setEnabled(false);
    }
  }

  /**
   * Necha uzivatele vybrat soubor. Potom z nej nacte programovanou gramatiku
   * a jeji pravidla vyplni do seznamu pravidel. Prepise prave editovanou 
   * gramatiku nove nactenou.
   */
  public void readFile(){
    String nondef;
    JFileChooser chooser;
    if(derivationFrame.getDirectory()!=null){
      chooser = new JFileChooser(  //Nastavi adresar, ve kterem byl naposledy
        derivationFrame.getDirectory()); // vybran soubor
    }
    else {
      chooser = new JFileChooser();
    }
    chooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
    chooser.setApproveButtonToolTipText("Nacte gramatiku z vybraneho souboru");
    chooser.setDialogTitle("Vyberte soubor s gramatikou");
    chooser.setApproveButtonText("Otevrit");
    chooser.setDialogType(JFileChooser.OPEN_DIALOG);
    int returnVal=chooser.showOpenDialog(this); //Provede vyber souboru
    if(returnVal==JFileChooser.APPROVE_OPTION){
      derivationFrame.setDirectory(chooser.getSelectedFile().getParentFile());
      PrgGrammarFile pgf = new PrgGrammarFile(chooser.getSelectedFile());
      try{
        this.grammar=pgf.readGrammar(); //Nacteni gramatiky
      }
      catch(java.text.ParseException e){ 
        this.error(e);  //Chyba formatu souboru
      }
      catch(java.io.FileNotFoundException e){ 
        this.error(ErrorFrame.FILE_NOT_FOUND); //Soubor nenalezen
      }
      catch(java.io.IOException e){  //Chyba pri cteni souboru
        this.error(ErrorFrame.PARSE_ERROR);
      }
 
      this.updateList();  //Opravi seznam pravidel
      updateSymbols();   //Zmeni zobrazeny seznam terminalu a neterminalu
      if(this.grammar.getStart()!=null){
        this.startField.setText(this.grammar.getStart().toString());
      }
    }
  }

  /**
   * Zapise prave editovanou gramatiku do souboru. Soubor necha vybrat 
   * uzivateli dialogem. Pred ulozenim musi byt vyplnen pocatecni neterminal
   * a vsechna pravidla uvedena v polich uspechu a neuspechu musi byt 
   * definovana.
   */
  private void writeFile(){
    JFileChooser chooser;
    if(!this.testGrammar()){
       return;
    }
    if(derivationFrame.getDirectory()!=null){
      chooser = new JFileChooser(
        derivationFrame.getDirectory()); //Nastavi naposledy uzivany adresar
    }
    else {
      chooser = new JFileChooser();
    }
    chooser.setDialogTitle("Vyberte soubor pro ulozeni gramatiky");
    chooser.setApproveButtonToolTipText("Ulozi gramatiku do vybraneho souboru");
    chooser.setDialogType(JFileChooser.SAVE_DIALOG);
    chooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
    int returnVal=chooser.showSaveDialog(this); //Umozni vyber souboru
    if(returnVal==JFileChooser.APPROVE_OPTION){
      derivationFrame.setDirectory(chooser.getSelectedFile().getParentFile());
      PrgGrammarFile pgf = new PrgGrammarFile(chooser.getSelectedFile());
      try{
        pgf.writeGrammar(this.grammar);  //Zapsani do souboru
      }
      catch(java.io.IOException e){   
        this.error(ErrorFrame.IO_ERROR); //Chyba pri zapisu do souboru
      }
    }
  }

  /**
   * Zmeni gramatiku. Podle nove gramatiky nastavi seznam pravidel a pocatecni
   * neterminal v prvcich okna
   *
   * @param g Nova gramatika
   */
  public void refreshGrammar(Grammar g){
    this.grammar=(ProgrammedGrammar) g;
    this.updateList();
    updateSymbols();
    this.startField.setText(this.grammar.getStart().toString());
  }

  /**
   * Slouzi ke zjisteni, ze bylo uzavreno okno s chybovym hlasenim. Podle 
   * chyby zobrazene v chybovem okne pozna, kam ma nastavit kurzor.
   *
   * @param message Cislo chybove hlasky, ktera byla zobrazena.
   */
  public void takeControl(int message){
    this.setEnabled(true);
    switch(message){
      case ErrorFrame.NO_LABEL:
      case ErrorFrame.EXIST_LABEL:
        this.labelField.requestFocus();
        break;
      case ErrorFrame.NO_LEFT:
      case ErrorFrame.LONG_LEFT:
      case ErrorFrame.WRONG_FORMAT:
        this.leftField.requestFocus();
        break;
      case ErrorFrame.WRONG_FORMAT_TN:
        this.rightField.requestFocus();
        break;
      case ErrorFrame.NO_START:
      case ErrorFrame.LONG_START:
      case ErrorFrame.WRONG_FORMAT_START:
        this.startField.requestFocus();
        break;
      case ErrorFrame.PARSE_ERROR:
        break;
    }
    this.show();
  }

  /** 
   * Zobrazi okno s chybovou hlaskou o spatnem formatu souboru.
   *
   * @param e Vyjimka generovana pri spatnem formatu souboru. Obsahuje
   * text chyby a radek, na kterem chyba nastala.
   */
  private void error(java.text.ParseException e){
    errorFrame=new ErrorFrame(this);
    this.setEnabled(false);
    errorFrame.show();
    StringBuffer buf=new StringBuffer();
    buf.append(e.getErrorOffset());
    buf.append(": ");
    buf.append(e.getMessage());
    errorFrame.setMessage(ErrorFrame.PARSE_ERROR,buf.toString());
  }

  /**
   * Slouzi k zobrazeni chybove hlasky s urcenym cislem.
   *
   * @param e Cislo chybove hlasky, ktera se ma zobrazit
   */
  private void error(int e){
    errorFrame=new ErrorFrame(this);
    this.setEnabled(false);
    errorFrame.show();
    errorFrame.setMessage(e);
  }

  /**
   * Prekontroluje textova pole, do kterych se vyplnuje pravidlo. Pokud je vse
   * v poradku, vytvori podle vyplnenych hodnot nove pravidlo. 
   *
   * @return Pravidlo odpovidajici vyplnenym hodnotam. Pri chybe vraci null.
   */
  protected PrgProduction readProduction(){
    int count;
    PrgProduction prod;
    StringTokenizer token;
    String labelStr;
    String left;
    String[] right;
    String[] succ;
    String[] fail;
    token=new StringTokenizer(this.labelField.getText(),
      PrgGrammarFile.delimiters);
    if(token.countTokens()==0){
      error(ErrorFrame.NO_LABEL);      //Nevyplneno oznaceni
      return null;
    }
    if(token.countTokens()>1){
      error(ErrorFrame.WRONG_LABEL);  //V poli pro oznaceni je vice retezcu
      return null;
    }
    labelStr=token.nextToken();
    if(this.grammar.existLabel(labelStr)){
      error(ErrorFrame.EXIST_LABEL);  //Oznaceni jiz bylo uzito drive
      return null;
    }
    token=new StringTokenizer(this.leftField.getText());
    if(token.countTokens()==0){
      error(ErrorFrame.NO_LEFT); //Nevyplnena leva strana pravidla
      return null;
    }
    else if(token.countTokens()>1){
      error(ErrorFrame.LONG_LEFT); //V leve strane je vice symbolu
      return null;
    }
    else{
      left=token.nextToken();
    }
    if(!Character.isUpperCase(left.charAt(0))){
      error(ErrorFrame.WRONG_FORMAT); //Na leve strane neni neterminal
      return null;
    }


    token=new StringTokenizer(this.rightField.getText());
    right=new String[token.countTokens()];
    for(int i=0; token.hasMoreElements(); i++){
      right[i]=token.nextToken();
      if(!Character.isLetter(right[i].charAt(0))){
        error(ErrorFrame.WRONG_FORMAT_TN); //Na prave strane nejsou jen spravne
        return null;                       //symboly
      }
    }

    token=new StringTokenizer(this.succField.getText(),
      PrgGrammarFile.delimiters);
    succ=new String[token.countTokens()];
    for(int i=0; token.hasMoreElements(); i++){
      succ[i]=token.nextToken();
    }

    token=new StringTokenizer(this.failField.getText(),
      PrgGrammarFile.delimiters);
    fail=new String[token.countTokens()];
    for(int i=0; token.hasMoreElements(); i++){
      fail[i]=token.nextToken();
    }

    Symbol r[]=new Symbol[right.length];
    for(int i=0; i<right.length;i++){ //Prida symboly z prave strany 
      r[i]=this.grammar.addSymbol(right[i]);
      if(r[i]==null){
        error(ErrorFrame.WRONG_FORMAT_TN);
        return null;
      }
    }
    Symbol l=this.grammar.addSymbol(left); //Prida symbol z leve strany
    if(l==null){
      error(ErrorFrame.WRONG_FORMAT);
      return null;
    }
    prod=new PrgProduction(labelStr, new SententialForm(l),
      new SententialForm(r));
    prod.setFailLabels(fail);  //Prevede oznaceni pravidel na pravidla
    prod.setSuccesLabels(succ);  //Prevede oznaceni pravidel na pravidla
    return prod;
  }

  /**
   * Vytvori seznam pravidel podle aktualni gramatiky.
   */
  protected void updateList(){
    Production p[];
    String s[];
    p=grammar.getProductions();
    s=new String[p.length];
    for(int i=0; i<p.length; i++){
      s[i]=p[i].toListItem();
    }
    prodList.setListData(s);
  }

  /**
   * Vymaze hodnoty z textovych poli pro editaci pravidla.
   */
  private void removeFields(){
    this.labelField.setText("");
    this.leftField.setText("");
    this.rightField.setText("");
    this.succField.setText("");
    this.failField.setText("");
  }

  /**
   * Precte nazev neterminalu z textoveho pole. Zkontroluje, jestli je jen 
   * jeden a jde o neterminal
   *
   * @return Nactene jmeno pocatecniho neterminalu. Pri chybe nebo neni-li 
   * vyplnen vraci null.
   */
  protected String readStart(){
    StringTokenizer token;
    String s;
    token=new StringTokenizer(this.startField.getText(),
      PrgGrammarFile.delimiters);
    if(token.countTokens()==0){
      error(ErrorFrame.NO_START); //Nevyplnen pocatecni neterminal
      return null;
    }
    else if(token.countTokens()>1){
      error(ErrorFrame.LONG_START); //Vyplneno vice symbolu
      return null;
    }
    else{
      s=token.nextToken();
    }
    if(!Character.isUpperCase(s.charAt(0))){
      error(ErrorFrame.WRONG_FORMAT_START); //Nejde o neterminal
      return null;
    }
    return s;
  }

  /**
   * Zmeni obsah textovych poli s terminaly a neterminaly podle aktualni 
   * gramatiky.
   */
  protected void updateSymbols(){
    Symbol[] symbols;
    StringBuffer sb=new StringBuffer("");
    symbols=grammar.getTerminals();
    for(int i=0;i<symbols.length;i++){
      if(i!=0){
        sb.append(", ");
      }
      sb.append(symbols[i].toString());
    }
    this.termField.setText(sb.toString());
    termField.setColumns(termField.getText().length());
    this.termScroll.setViewportView(this.termField);
    sb=new StringBuffer("");
    symbols=grammar.getNonterminals();
    for(int i=0;i<symbols.length;i++){
      if(i!=0){
        sb.append(", ");
      }
      sb.append(symbols[i].toString());
    }
    this.nontermField.setText(sb.toString());
    nontermField.setColumns(nontermField.getText().length());
    this.nontermScroll.setViewportView(this.nontermField);
    if(grammar.getStart()!=null && startField.getText().trim().length()==0){
      this.startField.setText(grammar.getStart().toString());
    }
  }

  /**
   * Zjisti, jestli gramatika splnuje zakladni pozadavky na ukonceni editace.
   * Musi byt vyplnen pocatecni neterminal a vsechna pravidla uvedena v polich 
   * uspechu a neuspechu musi existovat.
   *
   * @return Pokud jsou podminky splneny, vraci true. 
   */
  protected boolean testGrammar(){
    String nondef;
    if((nondef=this.readStart())!=null){
      grammar.setStart(grammar.addNonterminal(nondef));
    }
    else{
      return false;
    }
    if((nondef=grammar.labelToProds())!=null){
      errorFrame=new ErrorFrame(this);
      this.setEnabled(false);
      errorFrame.show();
      errorFrame.setMessage(ErrorFrame.NONDEF_LABEL,nondef);
      return false;
    }
    return true;
  }

  /**
   * Akce pri potvrzeni gramatiky. Otestuje gramatiku, nastavi ji pro derivaci
   * a zavre okno editace
   */
  protected void okButtonAction() {
    if(this.testGrammar()){
      this.derivationFrame.setGrammar(this.grammar);
      this.derivationFrame.setEnabled(true);
      this.hide();
    }
  }

  /**
   * Zavre okno editace bez ulozeni zmen v gramatice.
   */
  protected void stornoButtonAction() {
    this.derivationFrame.setEnabled(true);
    this.hide();
  }

  /**
   * Slouzi pro pridani pravidla do gramatiky. Nacte ho z textovych poli
   * pomoci funkce readProduction(). Pokud jde o prvni pravidlo, vyplni
   * jeho levou stranu jako pocatecni neterminal.
   */
  protected void addButtonAction() {
    Production prod;
    prod=this.readProduction();
    if(prod!=null){
      grammar.addProduction(prod);
      if(grammar.getStart()==null){
        grammar.setStart((Nonterminal) prod.getLeft().symbolAt(0));
      }
      this.updateList();
      updateSymbols();
      this.removeFields();
    }
  }

  /**
   * Slouzi pro odstraneni pravidla vybraneho v seznamu. Pokud neni vybrano 
   * zadne, neprovede metoda nic.
   */
  protected void deleteButtonAction() {
    int index = prodList.getSelectedIndex();
    if(index == -1){
      return;
    }
    grammar.removeProduction(index);
    updateList();
    updateSymbols();
  }

  /**
   * Umozni editaci pravidla. Pravidlo vybrane v seznamu vymaze z gramatiku, 
   * ale jeho hodnoty vyplni do textovych poli. Pokud neni vybrano zadne
   * pravidlo, neprovede metoda nic. 
   */
  protected void updateButtonAction() {
    PrgProduction prod;
    Production p[];
    int index = prodList.getSelectedIndex();
    if(index == -1){
      return;
    }
    p=grammar.getProductions();
    prod=(PrgProduction)p[index];
    this.labelField.setText(prod.getLabel());
    this.leftField.setText(prod.getLeft().toString());
    this.rightField.setText(prod.getRight().toString());
    this.succField.setText(prod.succesToString());
    this.failField.setText(prod.failToString());
    grammar.removeProduction(index);
    updateList();
    updateSymbols();
  }
}
